# joylink  device sdk  C 语言版 用户手册  


##  1. 概述及说明  
为简化设备开发流程，实现设备的快速接入，提供了C语言版的设备接入SDK。 在使用SDK之前，默认用户已经在物联平台完成设备创建等相关工作，并得到了设备标识、设备证书等所需要的信息。 



##  2. 名词解释  

为了方便用户能够快速的对使用手册进行约定，进而进行设备接入，对涉及到的相关名词进行简单说明。

**直连设备**：具备通过MQTT协议连接到物联平台的设备； 

**非直连设备**：不具备通过MQTT协议连接到物联平台的设备； 

**网关设备**：具备通过MQTT协议连接到物联平台，并且能够为非直连设备提供代理接入物联平台； 

**子设备**：这里指的时非直连设备； 

**属性**：一些可能表示状态，条件的独立参数，例如时间，名称，版本，温度，容量，开关等基础属性； 

**方法**：一个方法是一个具体的操作，有入参与出参，通过调用可以得到一个结果； 

**事件**：一些由条件触发的消息，由设备端主动发出；

**实例名 （key）**：物模型中一个属性实例，事件实例，方法实例，模型实例的唯一标识。需要在物模型中保持唯一；

**对象空间**：可选值 {“device”, "edge", "group"}， SDK目前支持的取值 {“device”}；

**服务实例名**：可选值 {NULL, "connection_agent", "ota", “config”}，SDK目前支持的取值 {NULL， “connection_agent”}；

**connection_agent**：一个内置的代理服务，对于网关设备此服务为必选，用于代理子设备完成相应的功能；直连设备可不需要该服务;  



##  3. 引用文件  

无 



##  4. 快速入门  

###  4.1. 必要说明  
SDK提供的例子是在Linux环境下，MQTT客户端使用的是mosquitto源码，因此在您进行例子编译之前，请确认您的开发环境具备以下条件，其它平台下SDK使用请参考下一节：
​[1] 系统为Linux系统；

[2] 系统安装已经安装 open-ssl-1.0.2 版本；



###  4.2. 编译及运行

####  4.2.1.  编译外部依赖库 mosquitto
外部依赖库编译一次即可，具体编译步骤如下：
​**Step 1：**
​                % cd external_libs/mosquitto
​**Step 2：**
​                % mkdir build
​**Step 3：**
​                % cd build
​**Step 4：**
​                % cmake ..
​**Step 5：**
​                % make



####  4.2.2.  编译SDK

SDK的编译步骤如下：
​**Step 1：**
​                % ./once.sh
​**Step 2：**
​                % make distclean 
​**Step 3：**
​               % make



####  4.2.2.  运行SDK例子

 运行一下脚本即可：
​               % ./run_test.sh



##  5. 开发使用说明  

这一章节将对SDK的API及使用方法进行详细说明。首先介绍的是硬件平台适配层，此章节建议您认真读完；其后进行API说明及详细说明，您可以根据自己的开发习惯，选择先看或者后续“回溯查看”；最后进行API调用顺序以及用户实现部分的相关说明及总结， 建议您认真阅读；



###  5.1. 硬件平台适配层  

为了适应不同的硬件平台，SDK抽象出了平台相关的接口函数定义，需要开发者根据自身的硬件平台进行适配，头文件路径为 pal/inc，文件列表及说明如下：

| 文件名 | 功能说明 | 备注 |
| :-----| :----| :----|
| joylink_log.h | 日志打印 | 无需适配 |
| joylink_memory.h | 内存相关接口 | 适配 |
| joylink_mqtt.h | MQTT客户端相关接口 | 适配 |
| joylink_stdint.h | 整型数定义 | 适配 |
| joylink_stdio.h | 标准输入输出 | 适配 |
| joylink_string.h | 字符串接口 | 适配 |
| joylink_thread.h | 线程接口 | 适配 |
| joylink_time.h | 时间相关接口 | 适配 |



####   5.1.1.  MQTT客户端适配说明

MQTT客户端需要适配的接口函数定义在pal/inc/ joylink_mqtt.h 头文件中，目前SDK提供了使用mosquitto进行适配的示例，具体参见 pal/src/joylink_mqtt.c 文件```__MQTT_MOSQUITTO_PAL__```宏定义中的代码。

如果您使用的硬件平台有自己的MQTT客户端，根据头文件抽象出的函数接口定义完成适配即可，需要的适配接口函数及说明如下：

| 函数 | 说明|
| :-----| :---- |
| jl_platform_mqtt_create | 创建客户端 |
| jl_platform_mqtt_destory | 销毁客户端 |
| jl_platform_mqtt_connect | 连接broker |
| jl_platform_mqtt_disconnect | 断开与broker的连接 |
| jl_platform_mqtt_yield | 循环执行函数 |
| jl_platform_mqtt_subscribe | 消息订阅函数 |
| jl_platform_mqtt_publish | 消息发布函数 |
| jl_platform_mqtt_set_cb_connect | 设置连接成功回调函数 |
| jl_platform_mqtt_set_cb_disconnect | 设置连接断开回调函数 |
| jl_platform_mqtt_set_cb_message | 设置消息回调函数 |



mosquitto适配 示例代码如下：

```c
/**
 * @brief   创建一个MQTT客户端
 *
 * @param[in] config: MQTT的设置参指针
 * @return 
 	MQTT客户端句柄
 * @see None.
 * @note None.
 */
jl_mqtt_t jl_platform_mqtt_create(jl_mqtt_config_t *config)
{
#ifdef __MQTT_MOSQUITTO_PAL__
    // mosquitto libarary initialise
    mosquitto_lib_init();

    // create mosquitto
    my_mosq = mosquitto_new(config->clientId, true, NULL);
    if(!my_mosq){
		return NULL;
	}
    
    // set options mqtt versions
    mosquitto_int_option(my_mosq, MOSQ_OPT_PROTOCOL_VERSION, MQTT_PROTOCOL_V311);

    // set options username and password
    if ((config->username != NULL) || (config->password != NULL)) {
        mosquitto_username_pw_set(my_mosq, config->username, config->password);
    }

    // set options tls
    if (!strcmp(config->protocol, "mqtt_tls")) {
        mosquitto_tls_set(my_mosq, config->cafile, NULL, config->cert, config->key, NULL);
    }

    // set options insecure
    if (config->insecure) {
        mosquitto_tls_insecure_set(my_mosq, true);
    }

    // set options max message inflight
    mosquitto_max_inflight_messages_set(my_mosq, 20);

    mosquitto_log_callback_set(my_mosq, mosq_log_callback);
	mosquitto_connect_v5_callback_set(my_mosq, mosq_connect_callback);
    mosquitto_disconnect_v5_callback_set(my_mosq, mosq_disconnect_callback);
	mosquitto_message_v5_callback_set(my_mosq, mosq_message_callback);
    
    return (jl_mqtt_t) my_mosq;
#else
    return NULL;
#endif
}
```



####   5.1.2.  其它函数接口适配说明
内存、字符串等硬件平台相关的接口适配实现，SDK中提供了Linux下的实现参考，源文件路径为 pal/src/，具体实现请参考宏 ```__LINUX_PAL__``` 内的代码。示例如下：

```c
 /**
 * @brief Allocates a block of size bytes of memory, returning a pointer to the beginning of the block.
 *
 * @param [in] size @n specify block size in bytes.
 * @return A pointer to the beginning of the block.
 * @see None.
 * @note Block value is indeterminate.
 */
void *jl_platform_malloc(uint32_t size)
{
#ifdef __LINUX_PAL__
    return malloc(size);
#else
    return NULL;
#endif
}
```



###  5.2.  SDK API 说明

SDK对外提供的头文件及功能如下：

| 文件名 | 说明 |
| :-----| :----- |
| joylink.h | 返回值等相关定义 |
| joylink_protocol.h | 通讯消息结构体定义及一些相关API的定义 |
| joylink_sdk.h | 对外主流程API定义 |



SDK提供的API函数及功能说明如下：

| 函数名 | 说明 |
| :-----| :----- |
| joylink_sdk_config_create | 创建SDK配置信息 |
| joylink_sdk_config_destory | 销毁SDK配置信息 |
| joylink_sdk_create | 创建SDK |
| joylink_sdk_destory | 销毁SDK |
| joylink_sdk_initialise | SDK初始化 |
| joylink_sdk_connect | 连接京东云 |
| joylink_sdk_main_loop | 进入主循环，阻塞 |
| joylink_sdk_get_messageId | 获取消息Id |
| joylink_sdk_set_dev_cb_connect | 设置设备回调函数：连接成功 |
| joylink_sdk_set_dev_cb_disconnect | 设置设备回调函数：连接断开 |
| joylink_sdk_prop_version_check | 属性设置时，检测请求版本 |
| joylink_sdk_set_dev_cb_prop_set | 设置设备回调函数：属性设置 |
| joylink_sdk_dev_prop_set_response | 发送属性设置响应 |
| joylink_sdk_set_dev_cb_prop_get | 设置设备回调函数：属性获取 |
| joylink_sdk_dev_prop_get_response | 发送属性获取响应 |
| joylink_sdk_dev_prop_post | 设备属性上报 |
| joylink_sdk_set_dev_cb_func_call | 设置设备回调函数：方法调用 |
| joylink_sdk_dev_func_call_response | 发送方法调用响应 |
| joylink_sdk_dev_evt_online_status_post | 直连设备上线状态上报 |
| joylink_sdk_dev_evt_post | 设备事件上报 |
| joylink_sdk_dev_reg_request | 设备自动注册请求 |
| joylink_sdk_set_dev_cb_reg_res | 设置设备回调函数：自动注册响应 |
| joylink_sdk_dev_thing_model_post | 直连设备物模型上报 |
| joylink_sdk_set_dev_cb_thmd_post_res | 设置设备回调函数：物模型上报响应 |
| joylink_sdk_dev_ntp_request | 设备请求时间同步 |
| joylink_sdk_set_dev_ntp_req_res | 设置设备回调函数：时间同步响应 |
| joylink_sdk_set_dev_ota_cb_func_call | 设置OTA回调函数：方法调用 |
| joylink_sdk_dev_ota_func_call_response | 发送OTA方法调用响应 |
| joylink_sdk_dev_ota_state_changed_post | 设备ota状态变化事件上报 |
| joylink_sdk_dev_ota_version_changed_post | 设备ota版本号变化事件上报 |
| joylink_sdk_set_agent_cb_func_call | 设置连接代理回调函数：方法调用 |
| joylink_sdk_agent_func_call_response | 发送连接代理方法调用响应 |
| joylink_sdk_sub_dev_online_post | 代理子设备上线事件上报 |
| joylink_sdk_sub_dev_offline_post | 代理子设备下线事件上报 |
| joylink_sdk_set_sub_dev_cb_prop_set | 设置子设备回调：属性设置 |
| joylink_sdk_sub_dev_prop_set_response | 代理子设备发送属性设置响应 |
| joylink_sdk_set_sub_dev_cb_prop_get | 设置子设备回调：属性获取 |
| joylink_sdk_sub_dev_prop_get_response | 代理子设备发送属性获取响应 |
| joylink_sdk_set_sub_dev_cb_func_call | 设置子设备回调：方法调用 |
| joylink_sdk_sub_dev_func_call_response | 代理子设备发送方法调用响应 |
| joylink_sdk_sub_dev_prop_post | 代理子设备进行属性上报 |
| joylink_sdk_sub_dev_evt_post | 代理子设备进行事件上报 |
|  |  |
| joylink_proto_keyvalue_parse_bool | 解析bool值 |
| joylink_proto_keyvalue_pack_bool | 打包bool值 |
| joylink_proto_keyvalue_parse_int32 | 解析int32 |
| joylink_proto_keyvalue_pack_int32 | 打包int32 |
| joylink_proto_keyvalue_parse_double | 解析double |
| joylink_proto_keyvalue_pack_double | 打包double |
| joylink_proto_keyvalue_parse_string | 解析字符串 |
| joylink_proto_keyvalue_pack_string | 打包字符串 |
|  |  |
| joylink_proto_ota_create_func_res | ota方法调用无出参响应 |
| joylink_proto_ota_create_func_query_res | ota状态查询方法调出参响应 |
|  |  |
|  以下函数慎用：成员变量 指针类型的成员变量，必须全部是由malloc产生的，结构体必须初始化 ||
| joylink_proto_free_prop_set_res | 释放属性设置响应结构体成员变量资源 |
| joylink_proto_free_prop_get_res | 释放属性获取响应结构体成员变量资源 |
| joylink_proto_free_prop_post | 释放属性上报结构体成员变量资源 |
| joylink_proto_free_func_call_res | 释放方法调用响应结构体成员变量资源 |
| joylink_proto_free_evt_post | 释放事件上报结构体成员变量资源 |
|  |  |



###  5.3.  SDK API 详细说明

####  joylink_sdk_config_create

***原型***

```c
joylink_sdk_config_t *joylink_sdk_config_create();
```
***函数功能说明***
创建SDK配置结构体，并返回指向其的指针；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| 无 |  |  |  |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| NULL |  | 失败 |
| 非空 | joylink_sdk_config_t * | 成功 |

***参数附加说明***
```c
typedef struct {
    char      protocol[16];
    char      *hostname;
    int32_t   port;
    char      *deviceId;
    char      *username;
    char      *password;
    char      *cafile;
    char      *cert;
    char      *key;
    bool      insecure;
    char      *devpubkey;
    char      *devprtkey;
    char      *platpubkey;
} joylink_sdk_config_t;
```

> **protocol**：协议类型，目前取值{"mqtt_tls"}；
> **hostname**：物联平台broker主机地址；
> **port**：物联平台broker主机地址端口；
> **deviceId**：设备标识，在物联平台创建设备时生成；
> **username**：用户名，与设备标识一致；
> **password**：用户密码，暂时填空；
> **cafile、cert、key**：设备证书文件；；
> **insecure**：是否验证主机名，目前需要填 true；
> **devpubkey、devprtkey、platpubkey**：保留字段，填NULL；  

* * * *



####  joylink_sdk_config_destory

***原型***

```c
int32_t joylink_sdk_config_destory(joylink_sdk_config_t *config);
```
***函数功能说明***
销毁SDK配置结构体；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| config | joylink_sdk_config_t * | 输入 |  |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***

```c
typedef enum {
    JOYLINK_RET_SUCCESS   = 0,
    JOYLINK_RET_FAILED    = -1,
} JoylinkRetCode_E;
```
> **JOYLINK_RET_SUCCESS**：成功；
> **JOYLINK_RET_FAILED**：失败；

* * * *



####  joylink_sdk_create

***原型***

```c
struct joylink_sdk_t *joylink_sdk_create(joylink_sdk_config_t *config);
```
***函数功能说明***
创建SDK，并返回指向其的指针；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| config | joylink_sdk_config_t * | 输入 |  |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| NULL | | 失败 |
| 非空 | struct joylink_sdk_t * | 成功 |

* * * *



####  joylink_sdk_destory

***原型***

```c
int32_t joylink_sdk_destory(struct joylink_sdk_t *sdk);
```
***函数功能说明***
销毁SDK；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

* * * *



####  joylink_sdk_initialise

***原型***

```c
int32_t joylink_sdk_initialise(struct joylink_sdk_t *sdk);
```
***函数功能说明***
SDK初始化；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

* * * *



####  joylink_sdk_connect

***原型***

```c
int32_t joylink_sdk_connect(struct joylink_sdk_t *sdk);
```
***函数功能说明***
SDK连接物联平台Broker；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

* * * *



####  joylink_sdk_main_loop

***原型***

```c
int32_t joylink_sdk_main_loop(struct joylink_sdk_t *sdk);
```
***函数功能说明***
SDK进入主循环，阻塞性的函数；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

* * * *



####  joylink_sdk_get_messageId

***原型***

```c
char *joylink_sdk_get_messageId(char *prefix);
```
***函数功能说明***
用于产生消息Id；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| prefix | char * | 输入 | 自定义前缀，可以为空 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| NULL |  | 失败 |
| 非空 | char * | 成功 |

* * * *



####  joylink_sdk_set_dev_cb_connect

***原型***

```c
int32_t joylink_sdk_set_dev_cb_connect(struct joylink_sdk_t *sdk,
            joylink_sdk_callback_dev_connect cb_connect);
```
***函数功能说明***
设置设备回调函数：连接物联平台成功；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| cb_connect | joylink_sdk_callback_dev_connect | 输入 | 设备回调函数句柄 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***
```c
typedef int32_t (*joylink_sdk_callback_dev_connect) (struct joylink_sdk_t *);
```

* * * *



####  joylink_sdk_set_dev_cb_disconnect

***原型***

```c
int32_t joylink_sdk_set_dev_cb_disconnect(struct joylink_sdk_t *sdk,
            joylink_sdk_callback_dev_disconnect cb_disconnect);
```
***函数功能说明***
设置设备回调函数：与物联平台断开连接；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| cb_disconnect | joylink_sdk_callback_dev_disconnect | 输入 | 设备回调函数句柄 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***
```c
typedef int32_t (*joylink_sdk_callback_dev_disconnect) (struct joylink_sdk_t *);
```

* * * *



####  joylink_sdk_prop_version_check

***原型***

```c
int32_t joylink_sdk_prop_version_check(uint16_t cur_version, uint16_t req_version);
```
***函数功能说明***
校验属性设置请求的版本号，如果通过则更新属性，如果通过则放弃更新；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| cur_version | uint16_t | 输入 | 设备属性当前版本 |
| req_version | uint16_t | 输入 | 属性设置请求的版本号 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| 1 | int32_t | 通过 |
| 0 | int32_t | 不通过 |

* * * *



####  joylink_sdk_set_dev_cb_prop_set

***原型***

```c
int32_t joylink_sdk_set_dev_cb_prop_set(struct joylink_sdk_t *sdk,
            joylink_sdk_callback_dev_prop_set cb_prop_set);
```
***函数功能说明***
设置设备回调函数：属性设置；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| cb_prop_set | joylink_sdk_callback_dev_prop_set | 输入 | 设备回调函数句柄 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***

```c
typedef struct {
    char        *key;
    char        *value;
} JoyProtoKeyValue_t;

typedef struct {
    char                *deviceId;
    uint32_t            timestamp;
    char                *messageId;
    uint16_t            version;
    int32_t             prop_num;
    JoyProtoKeyValue_t  **properties;
} JoyProtoPropSet_t;

typedef int32_t (*joylink_sdk_callback_dev_prop_set) (struct joylink_sdk_t *,
                              char *obj_name, char *service_key, JoyProtoPropSet_t *);
```
> **obj_name**：对象空间；
> **service_key**：服务实例名；
> **deviceId**：设备标识；
> **timestamp**：时间戳；
> **messageId**：消息Id；
> **version**：版本号；
> **prop_num**：请求报文中属性的个数；
> **properties**：属性数组，个数是prop_num；
> **key**：属性的关键字；
> **value**：属性的值，为json字符串；

* * * *



####  joylink_sdk_dev_prop_set_response

***原型***

```c
int32_t joylink_sdk_dev_prop_set_response(struct joylink_sdk_t *sdk,
            char *obj_name, char *service_key, JoyProtoPropSetRes_t *in_res);
```
***函数功能说明***
发送设备属性设置响应；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| obj_name | char * | 输入 | 对象空间 |
| service_key | char * | 输入 | 服务实例名 |
| in_res | JoyProtoPropSetRes_t * | 输入 | 属性设置响应结构体指针 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***
```c
typedef struct {
    char                *deviceId;
    uint32_t            timestamp;
    char                *messageId;
    uint16_t            version;
    int32_t             code;
    char                *message;
    int32_t             prop_num;
    JoyProtoKeyValue_t  **properties;
} JoyProtoPropSetRes_t;
```
> **code**：返回状态码；
> **message**：返回状态描述；

* * * *



####  joylink_sdk_set_dev_cb_prop_get

***原型***

```c
int32_t joylink_sdk_set_dev_cb_prop_get(struct joylink_sdk_t *sdk,
            joylink_sdk_callback_dev_prop_get cb_prop_get);
```
***函数功能说明***
设置设备回调函数：属性获取；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| cb_prop_get | joylink_sdk_callback_dev_prop_get | 输入 | 设备回调函数句柄 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***
```c
typedef struct {
    char                *deviceId;
    uint32_t            timestamp;
    char                *messageId;
    int32_t             prop_num;
    JoyProtoKeyValue_t  **properties;
} JoyProtoPropGet_t;

typedef int32_t (*joylink_sdk_callback_dev_prop_get) (struct joylink_sdk_t *, char *obj_name, char *service_key, JoyProtoPropGet_t *);
```

* * * *



####  joylink_sdk_dev_prop_get_response

***原型***

```c
int32_t joylink_sdk_dev_prop_get_response(struct joylink_sdk_t *sdk,
            char *obj_name, char *service_key, JoyProtoPropGetRes_t *in_res);
```
***函数功能说明***
发布属性获取响应；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| obj_name | char * | 输入 | 对象空间 |
| service_key | char * | 输入 | 服务实例名 |
| in_res | JoyProtoPropGetRes_t * | 输入 | 属性获取响应结构体指针 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***
```c
typedef JoyProtoPropSetRes_t JoyProtoPropGetRes_t;
```

* * * *



####  joylink_sdk_dev_prop_post

***原型***

```c
int32_t joylink_sdk_dev_prop_post(struct joylink_sdk_t *sdk,
            char *obj_name, char *service_key, JoyProtoPropPost_t *in_post);
```
***函数功能说明***
上报设备属性；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| obj_name | char * | 输入 | 对象空间 |
| service_key | char * | 输入 | 服务实例名 |
| in_post | JoyProtoPropPost_t * | 输入 | 属性上报结构体指针 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***
```c
typedef JoyProtoPropSet_t    JoyProtoPropPost_t;
```

* * * *



####  joylink_sdk_set_dev_cb_func_call

***原型***

```c
int32_t joylink_sdk_set_dev_cb_func_call(struct joylink_sdk_t *sdk,
            joylink_sdk_callback_dev_func_call cb_func_call);
```
***函数功能说明***
设置设备回调函数：方法调用；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| cb_func_call | joylink_sdk_callback_dev_func_call | 输入 | 设备回调函数句柄 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***
```c
typedef struct {
    char                *key;
    int32_t             in_num;
    JoyProtoKeyValue_t  **in;
} JoyProtoFuncCallFunc_t;

typedef struct {
    char                    *deviceId;
    uint32_t                timestamp;
    char                    *messageId;
    int32_t                 func_num;
    JoyProtoFuncCallFunc_t  **functions;
} JoyProtoFuncCall_t;

typedef int32_t (*joylink_sdk_callback_dev_func_call) (struct joylink_sdk_t *, char *obj_name, char *service_key, JoyProtoFuncCall_t *);
```
> **func_num**：消息报文中方法的个数；
> **functions**：方法数组，个数是 func_num；
> **JoyProtoFuncCallFunc_t->key**：方法的关键字；
> **JoyProtoFuncCallFunc_t->in_num**：方法的入参个数；
> **JoyProtoFuncCallFunc_t->in**：方法的入参，个数是 in_num；

* * * *



####  joylink_sdk_dev_func_call_response

***原型***

```c
int32_t joylink_sdk_dev_func_call_response(struct joylink_sdk_t *sdk,
            char *obj_name, char *service_key, JoyProtoFuncCallRes_t *in_res);
```
***函数功能说明***
发布方法调用响应；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| obj_name | char * | 输入 | 对象空间 |
| service_key | char * | 输入 | 服务实例名 |
| in_res | JoyProtoFuncCallRes_t * | 输入 | 方法调用响应结构体指针 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***
```c
typedef struct {
    char                *key;
    int32_t             out_num;
    JoyProtoKeyValue_t  **out;
} JoyProtoFuncCallResFunc_t;

typedef struct {
    char                        *deviceId;
    uint32_t                    timestamp;
    char                        *messageId;
    int32_t                     code;
    char                        *message;
    int32_t                     func_num;
    JoyProtoFuncCallResFunc_t   **functions;
} JoyProtoFuncCallRes_t;
```

* * * *



####  joylink_sdk_dev_evt_online_status_post

***原型***

```c
int32_t joylink_sdk_dev_evt_online_status_post(
              struct joylink_sdk_t *sdk,
              char *obj_name,
              char *service_key,
              JoyProtoEvtOnlineStatus_t *in_status);
```
***函数功能说明***
直连设备上线状态上报；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| obj_name | char * | 输入 | 对象空间 |
| service_key | char * | 输入 | 服务实例名 |
| in_status | JoyProtoEvtOnlineStatus_t * | 输入 | 在线状态结构体指针 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***
```c
typedef struct {
    char                    *deviceId;
    uint32_t                timestamp;
    char                    *messageId;
    bool                    is_online;
    JoyProtoThingModel_t    thing_model;
} JoyProtoEvtOnlineStatus_t;
```

* * * *



####  joylink_sdk_dev_evt_post

***原型***

```c
int32_t joylink_sdk_dev_evt_post(
            struct joylink_sdk_t *sdk,
            char *obj_name,
            char *service_key,
            JoyProtoEvtPost_t *in_post);
```
***函数功能说明***
事件上报；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| obj_name | char * | 输入 | 对象空间 |
| service_key | char * | 输入 | 服务实例名 |
| in_post | JoyProtoEvtPost_t * | 输入 | 事件结构体指针 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***
```c
typedef struct {
    char                *key;
    int32_t             param_num;
    JoyProtoKeyValue_t  **parameters;    
}JoyProtoEvtPostEvt_t;

typedef struct {
    char                    *deviceId;
    uint32_t                timestamp;
    char                    *messageId;
    int32_t                 evt_num;
    JoyProtoEvtPostEvt_t    **events;
} JoyProtoEvtPost_t;
```

* * * *



####  joylink_sdk_dev_reg_request

***原型***

```c
int32_t joylink_sdk_dev_reg_request(
            struct joylink_sdk_t *sdk,
            char *obj_name,
            char *service_key,
            JoyProtoRegReq_t *in_req);
```
***函数功能说明***
自动注册请求；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| obj_name | char * | 输入 | 对象空间 |
| service_key | char * | 输入 | 服务实例名 |
| in_req | JoyProtoRegReq_t * | 输入 | 自动注册结构体指针 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***
```c
typedef struct {
    char        *key;
    char        *value;
    char        *desc;
} JoyProtoRegReqDevExt_t;
typedef struct {
    char                    *hardwareId;
    char                    *vendor;
    char                    *protocol;
    char                    *product;
    char                    *model;
    int32_t                 ext_num;
    JoyProtoRegReqDevExt_t  **extensions;
} JoyProtoRegReqDev_t;
typedef struct {
    char                    *deviceId;
    uint32_t                timestamp;
    char                    *messageId;
    int32_t                 dev_num;
    JoyProtoRegReqDev_t     **devices;
    char                    cerType[8];
} JoyProtoRegReq_t;

```

* * * *



####  joylink_sdk_set_dev_cb_reg_res

***原型***

```c
int32_t joylink_sdk_set_dev_cb_reg_res(struct joylink_sdk_t *sdk,
            joylink_sdk_callback_dev_reg_res cb_reg_res);
```
***函数功能说明***
设置设备回调函数：自动注册响应；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| cb_reg_res | joylink_sdk_callback_dev_reg_res | 输入 | 设备回调函数句柄 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***
```c
typedef struct {
    char        *name;
    char        *type;
    char        *content;
} JoyProtoRegReqResDevCfg_t;

typedef struct {
    char                        *deviceId;
    char                        *hardwareId;
    uint32_t                    cfg_num;
    JoyProtoRegReqResDevCfg_t   **config;
} JoyProtoRegReqResDev_t;

typedef struct {
    char                        *deviceId;
    uint32_t                    timestamp;
    char                        *messageId;
    int32_t                     code;
    char                        *message;
    uint32_t                    dev_num;
    JoyProtoRegReqResDev_t      **devices;
} JoyProtoRegReqRes_t;

typedef int32_t (*joylink_sdk_callback_dev_reg_res) (struct joylink_sdk_t *, char *obj_name, char *service_key, JoyProtoRegReqRes_t *);
```

* * * *



####  joylink_sdk_dev_thing_model_post

***原型***

```c
int32_t joylink_sdk_dev_thing_model_post(
            struct joylink_sdk_t *sdk,
            char *obj_name,
            char *service_key,
            JoyProtoThingModelPost_t *in_post);
```
***函数功能说明***
物模型上报；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| obj_name | char * | 输入 | 对象空间 |
| service_key | char * | 输入 | 服务实例名 |
| in_post | JoyProtoThingModelPost_t * | 输入 | 物模型上报结构体指针 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***
```c
typedef struct {
    char   *id;
    char   *version;
} JoyProtoThingModel_t;

typedef struct {
    char                    *deviceId;
    uint32_t                timestamp;
    char                    *messageId;
    JoyProtoThingModel_t    thing_model;
} JoyProtoThingModelPost_t;
```
* * * *



####  joylink_sdk_set_dev_cb_thmd_post_res

***原型***

```c
int32_t joylink_sdk_set_dev_cb_thmd_post_res(struct joylink_sdk_t *sdk,
            joylink_sdk_callback_dev_thmd_post_res cb_thmd_post_res);
```
***函数功能说明***
设置设备回调函数：物模型上报响应；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| cb_thmd_post_res | joylink_sdk_callback_dev_thmd_post_res | 输入 | 设备回调函数句柄 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***
```c
typedef struct {
    char                *deviceId;
    uint32_t            timestamp;
    char                *messageId;
    int32_t             code;
    char                *message;
} JoyProtoThingModelPostRes_t;
```
* * * *



####  joylink_sdk_dev_ntp_request

***原型***

```c
int32_t joylink_sdk_dev_ntp_request(struct joylink_sdk_t *sdk);
```
***函数功能说明***
设备请求时间同步

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

* * * *



####  joylink_sdk_set_dev_ntp_req_res

***原型***

```c
int32_t joylink_sdk_set_dev_ntp_req_res(struct joylink_sdk_t *sdk,
            joylink_sdk_callback_dev_ntp_req_res cb_ntp_req_res);
```
***函数功能说明***
设置设备回调函数：设备时间同步响应；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| cb_ntp_req_res | joylink_sdk_callback_dev_ntp_req_res | 输入 | 设备回调函数句柄 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***
```c
typedef struct {
    uint32_t second;
    uint32_t ms;
} jl_time_stamp_t;
```
* * * *



####  joylink_sdk_set_dev_ota_cb_func_call

***原型***

```c
int32_t joylink_sdk_set_dev_ota_cb_func_call(struct joylink_sdk_t *sdk,
            joylink_sdk_callback_dev_ota_func_call cb_func_call);
```

***函数功能说明***
设置OTA服务回调函数：方法调用；

***参数说明***

| 参数         | 类型                                   | 输入输出 | 说明             |
| :----------- | :------------------------------------- | :------- | :--------------- |
| sdk          | struct joylink_sdk_t *                 | 输入     | SDK句柄          |
| cb_func_call | joylink_sdk_callback_dev_ota_func_call | 输入     | 设备回调函数句柄 |

***返回值***

| 值               | 类型             | 说明                       |
| :--------------- | :--------------- | :------------------------- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***

```c
typedef int32_t (*joylink_sdk_callback_dev_ota_func_call) (struct joylink_sdk_t *, char *, JoyProtoFuncCall_t *);
```

* * * *



####  joylink_sdk_dev_ota_func_call_response

***原型***

```c
int32_t joylink_sdk_dev_ota_func_call_response(struct joylink_sdk_t *sdk, char *obj_name, JoyProtoFuncCallRes_t *in_res);
```

***函数功能说明***
发布OTA服务方法调用响应；

***参数说明***

| 参数     | 类型                    | 输入输出 | 说明                   |
| :------- | :---------------------- | :------- | :--------------------- |
| sdk      | struct joylink_sdk_t *  | 输入     | SDK句柄                |
| obj_name | char *                  | 输入     | 对象空间               |
| in_res   | JoyProtoFuncCallRes_t * | 输入     | 方法调用响应结构体指针 |

***返回值***

| 值               | 类型             | 说明                       |
| :--------------- | :--------------- | :------------------------- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

* * * *



####  joylink_sdk_dev_ota_state_changed_post

***原型***

```c
int32_t joylink_sdk_dev_ota_state_changed_post(
                   struct joylink_sdk_t *sdk,
                   char *obj_name,
                   char *task_id,
                   JOY_PROTO_OTA_STATE_T state,
                   JOY_PROTO_OTA_ERROR_T e_code,
                   int8_t progress);
```
***函数功能说明***
设备ota升级过程中，如果状态出现变化，可以调用改API将状态进行上报；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| obj_name | char * | 输入 | 对象空间 |
| task_id | char * | 输入 | 任务标识 |
| state | JOY_PROTO_OTA_STATE_T | 输入 | 当前状态 |
| e_code | JOY_PROTO_OTA_ERROR_T | 输入 | 错误码 |
| progress | int8_t | 输入 | 进度百分比 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***
```c
/**
 * @brief  ota 当前状态枚举
 */
typedef enum {
    JOY_OTA_ST_PREPARE,           // 正在准备
    JOY_OTA_ST_CF_DOWNLOAD,       // 等待用户确认下载
    JOY_OTA_ST_DOWNLOADIND,       // 正在下载
    JOY_OTA_ST_DOWNLOAD_DONE,     // 下载完成
    JOY_OTA_ST_CF_INSTALL,        // 等待用户确认安装
    JOY_OTA_ST_INSTALLING,        // 正在安装
    JOY_OTA_ST_INSTALL_DONE,      // 完成
    JOY_OTA_ST_FAILURE,           // 失败
} JOY_PROTO_OTA_STATE_E;
typedef int32_t  JOY_PROTO_OTA_STATE_T;

/**
 * @brief  ota 状态错误码
 */
typedef enum {
    JOY_OTA_ERROR_NONE,             // 无异常
    JOY_OTA_ERROR_TASK_CL,          // OTA任务已取消
    JOY_OTA_ERROR_DOWNLOAD_CL,      // 用户取消下载
    JOY_OTA_ERROR_INSTALL_CL,       // 用户取消安装
    JOY_OTA_ERROR_DPG_VER_FAIL,     // 差分包基包验证失败
    JOY_OTA_ERROR_DOWNLOAD_FAIL,    // 下载失败
    JOY_OTA_ERROR_SIG_VER_FAIL,     // 签名验证失败
    JOY_OTA_ERROR_DPG_RVT_FAIL,     // 差分包还原失败
    JOY_OTA_ERROR_INSTALL_FAIL,     // 安装失败
} JOY_PROTO_OTA_ERROR_E;
typedef int32_t  JOY_PROTO_OTA_ERROR_T;
```
* * * *



####  joylink_sdk_dev_ota_version_changed_post

***原型***

```c
int32_t joylink_sdk_dev_ota_version_changed_post(struct joylink_sdk_t *sdk, char *obj_name, char *object_id, char *version);
```
***函数功能说明***
设备ota版本号出现变化的时候，需要通过该API上报给物联平台；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| obj_name | char * | 输入 | 对象空间 |
| object_id | char * | 输入 | 设备标识 |
| version | char * | 输入 | 当前ota版本号 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

* * * *



####  joylink_proto_ota_create_func_res

***原型***

```c
JoyProtoFuncCallResFunc_t *joylink_proto_ota_create_func_res(char *func_key);
```
***函数功能说明***
生成方法调用响应 消息体 functions 成员变量；此函数仅适用于ota内置的无出参的方法调用；{"ota.ota", "ota.cancel-task", "ota.exec-task"}

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| func_key | char * | 输入 | 方法调用实例名 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| NULL | JoyProtoFuncCallResFunc_t * | 失败 |
| 非空 | JoyProtoFuncCallResFunc_t * | 成功 |

* * * *



####  joylink_proto_ota_create_func_query_res

***原型***

```c
JoyProtoFuncCallResFunc_t *joylink_proto_ota_create_func_query_res(
                                                      char *func_key,
                                                      char *task_id,
                                                      int32_t state,
                                                      int8_t progress);
```
***函数功能说明***
生成方法调用响应 消息体 functions 成员变量；此函数仅适用于ota内置的状态查询"ota.query-ota-state"方法调用。

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| func_key | char * | 输入 | 方法调用实例名 |
| task_id | char * | 输入 | ota任务标识 |
| state | int32_t | 输入 | ota当前状态 |
| progress | int8_t | 输入 | ota任务进度百分比 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| NULL | JoyProtoFuncCallResFunc_t * | 失败 |
| 非空 | JoyProtoFuncCallResFunc_t * | 成功 |
***参数附加说明***
> API 入参 state 为 JOY_PROTO_OTA_STATE_T；

* * * *



####  joylink_sdk_set_agent_cb_func_call

***原型***

```c
int32_t joylink_sdk_set_agent_cb_func_call(struct joylink_sdk_t *sdk,
            joylink_sdk_callback_agent_func_call cb_func_call);
```

***函数功能说明***
设置连接代理服务回调函数：方法调用；

***参数说明***

| 参数         | 类型                                 | 输入输出 | 说明             |
| :----------- | :----------------------------------- | :------- | :--------------- |
| sdk          | struct joylink_sdk_t *               | 输入     | SDK句柄          |
| cb_func_call | joylink_sdk_callback_agent_func_call | 输入     | 设备回调函数句柄 |

***返回值***

| 值               | 类型             | 说明                       |
| :--------------- | :--------------- | :------------------------- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***

```c
typedef int32_t (*joylink_sdk_callback_agent_func_call) (struct joylink_sdk_t *, JoyProtoFuncCall_t *);
```

* * * *



####  joylink_sdk_agent_func_call_response

***原型***

```c
int32_t joylink_sdk_agent_func_call_response(struct joylink_sdk_t *sdk, JoyProtoFuncCallRes_t *in_res);
```

***函数功能说明***
发送连接代理服务方法调用的响应。

***参数说明***

| 参数   | 类型                    | 输入输出 | 说明                   |
| :----- | :---------------------- | :------- | :--------------------- |
| sdk    | struct joylink_sdk_t *  | 输入     | SDK句柄                |
| in_res | JoyProtoFuncCallRes_t * | 输入     | 方法调用响应结构体指针 |

***返回值***

| 值               | 类型             | 说明                       |
| :--------------- | :--------------- | :------------------------- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

* * * *



####  joylink_sdk_sub_dev_online_post

***原型***

```c
int32_t joylink_sdk_sub_dev_online_post(struct joylink_sdk_t *sdk, char *obj_name, JoyProtoSubDevice_t *sub_dev);
```
***函数功能说明***
代理子设备进行上线消息上报；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| obj_name | char * | 输入 | 对象空间 |
| sub_dev | JoyProtoSubDevice_t * | 输入 | 子设备信息 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***
```c
typedef struct {
    char    *name;				// 设备名称
    char    *deviceId;			// 设备id
    char    *description;		// 设备描述
    char    *modelId;			// 设备物模型id
    char    *serviceName;		// 设备驱动名
    char    *hardwareId;		// 设备的唯一硬件标识，通常是设备MAC
    char    *protocolName;		// 设备通讯协议
    char    *protocolSpec;		// 设备通讯协议的详细说明，json对像字符串
}JoyProtoSubDevice_t;
```
* * * *



####  joylink_sdk_sub_dev_offline_post

***原型***

```c
int32_t joylink_sdk_sub_dev_offline_post(struct joylink_sdk_t *sdk, char *obj_name, JoyProtoSubDevice_t *sub_dev);
```
***函数功能说明***
代理子设备进行下线消息上报；

***参数说明***

| 参数 | 类型 | 输入输出 | 说明 |
| :-----| :----- |:----- |:----- |
| sdk | struct joylink_sdk_t * | 输入 | SDK句柄 |
| obj_name | char * | 输入 | 对象空间 |
| sub_dev | JoyProtoSubDevice_t * | 输入 | 子设备信息 |

***返回值***

| 值 | 类型 | 说明 |
| :-----| :----- |:----- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

* * * *



####  joylink_sdk_set_sub_dev_cb_prop_set

***原型***

```c
int32_t joylink_sdk_set_sub_dev_cb_prop_set(struct joylink_sdk_t *sdk,
            joylink_sdk_callback_sub_dev_prop_set cb_prop_set);
```

***函数功能说明***
设置子设备回调函数：属性设置；

***参数说明***

| 参数        | 类型                                  | 输入输出 | 说明             |
| :---------- | :------------------------------------ | :------- | :--------------- |
| sdk         | struct joylink_sdk_t *                | 输入     | SDK句柄          |
| cb_prop_set | joylink_sdk_callback_sub_dev_prop_set | 输入     | 设备回调函数句柄 |

***返回值***

| 值               | 类型             | 说明                       |
| :--------------- | :--------------- | :------------------------- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***

```c
typedef int32_t (*joylink_sdk_callback_sub_dev_prop_set) (struct joylink_sdk_t *, JoyProtoPropSet_t *);
```

* * * *



####  joylink_sdk_sub_dev_prop_set_response

***原型***

```c
int32_t joylink_sdk_sub_dev_prop_set_response(struct joylink_sdk_t *sdk, JoyProtoPropSetRes_t *in_res);
```

***函数功能说明***
代理子设备发送设备属性设置响应；

***参数说明***

| 参数   | 类型                   | 输入输出 | 说明                   |
| :----- | :--------------------- | :------- | :--------------------- |
| sdk    | struct joylink_sdk_t * | 输入     | SDK句柄                |
| in_res | JoyProtoPropSetRes_t * | 输入     | 属性设置响应结构体指针 |

***返回值***

| 值               | 类型             | 说明                       |
| :--------------- | :--------------- | :------------------------- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

**附加说明**

in_res 中的设备标识为**子设备**的设备标识

* * * *



####  joylink_sdk_set_sub_dev_cb_prop_get

***原型***

```c
int32_t joylink_sdk_set_sub_dev_cb_prop_get(struct joylink_sdk_t *sdk,
            joylink_sdk_callback_sub_dev_prop_get cb_prop_get);
```

***函数功能说明***
设置子设备回调函数：属性获取；

***参数说明***

| 参数        | 类型                                  | 输入输出 | 说明             |
| :---------- | :------------------------------------ | :------- | :--------------- |
| sdk         | struct joylink_sdk_t *                | 输入     | SDK句柄          |
| cb_prop_get | joylink_sdk_callback_sub_dev_prop_get | 输入     | 设备回调函数句柄 |

***返回值***

| 值               | 类型             | 说明                       |
| :--------------- | :--------------- | :------------------------- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***

```c
typedef int32_t (*joylink_sdk_callback_sub_dev_prop_get) (struct joylink_sdk_t *, JoyProtoPropGet_t *);
```

* * * *



####  joylink_sdk_sub_dev_prop_get_response

***原型***

```c
int32_t joylink_sdk_sub_dev_prop_get_response(struct joylink_sdk_t *sdk, JoyProtoPropGetRes_t *in_res);
```

***函数功能说明***
代理子设备发布属性获取响应；

***参数说明***

| 参数   | 类型                   | 输入输出 | 说明                   |
| :----- | :--------------------- | :------- | :--------------------- |
| sdk    | struct joylink_sdk_t * | 输入     | SDK句柄                |
| in_res | JoyProtoPropGetRes_t * | 输入     | 属性获取响应结构体指针 |

***返回值***

| 值               | 类型             | 说明                       |
| :--------------- | :--------------- | :------------------------- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

**附加说明**

in_res 结构体中的设备标识为**子设备**的设备标识。

* * * *



####  joylink_sdk_set_sub_dev_cb_func_call

***原型***

```c
int32_t joylink_sdk_set_sub_dev_cb_func_call(struct joylink_sdk_t *sdk,
            joylink_sdk_callback_sub_dev_func_call cb_func_call);
```

***函数功能说明***
设置子设备回调函数：方法调用；

***参数说明***

| 参数         | 类型                                   | 输入输出 | 说明             |
| :----------- | :------------------------------------- | :------- | :--------------- |
| sdk          | struct joylink_sdk_t *                 | 输入     | SDK句柄          |
| cb_func_call | joylink_sdk_callback_sub_dev_func_call | 输入     | 设备回调函数句柄 |

***返回值***

| 值               | 类型             | 说明                       |
| :--------------- | :--------------- | :------------------------- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

***参数附加说明***

```c
typedef int32_t (*joylink_sdk_callback_sub_dev_func_call) (struct joylink_sdk_t *, JoyProtoFuncCall_t *);
```

* * * *



####  joylink_sdk_sub_dev_func_call_response

***原型***

```c
int32_t joylink_sdk_sub_dev_func_call_response(struct joylink_sdk_t *sdk, JoyProtoFuncCallRes_t *in_res);
```

***函数功能说明***
代理子设备发布方法调用响应；

***参数说明***

| 参数   | 类型                    | 输入输出 | 说明                   |
| :----- | :---------------------- | :------- | :--------------------- |
| sdk    | struct joylink_sdk_t *  | 输入     | SDK句柄                |
| in_res | JoyProtoFuncCallRes_t * | 输入     | 方法调用响应结构体指针 |

***返回值***

| 值               | 类型             | 说明                       |
| :--------------- | :--------------- | :------------------------- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

**附加说明**

in_res 中的设备标识为**子设备**的设备标识。

* * * *



####  joylink_sdk_sub_dev_prop_post

***原型***

```c
int32_t joylink_sdk_sub_dev_prop_post(struct joylink_sdk_t *sdk, JoyProtoPropPost_t *in_post);
```

***函数功能说明***
代理子设备进行属性上报；

***参数说明***

| 参数    | 类型                   | 输入输出 | 说明               |
| :------ | :--------------------- | :------- | :----------------- |
| sdk     | struct joylink_sdk_t * | 输入     | SDK句柄            |
| in_post | JoyProtoPropPost_t *   | 输入     | 属性上报结构体指针 |

***返回值***

| 值               | 类型             | 说明                       |
| :--------------- | :--------------- | :------------------------- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

**附加说明**

in_post 结构体中的设备标识为**子设备**的设备标识。

* * * *



####  joylink_sdk_sub_dev_evt_post

***原型***

```c
int32_t joylink_sdk_sub_dev_evt_post(struct joylink_sdk_t *sdk, JoyProtoEvtPost_t *in_post);
```

***函数功能说明***
代理子设备进行事件上报；

***参数说明***

| 参数    | 类型                   | 输入输出 | 说明           |
| :------ | :--------------------- | :------- | :------------- |
| sdk     | struct joylink_sdk_t * | 输入     | SDK句柄        |
| in_post | JoyProtoEvtPost_t *    | 输入     | 事件结构体指针 |

***返回值***

| 值               | 类型             | 说明                       |
| :--------------- | :--------------- | :------------------------- |
| JoylinkRetCode_E | JoylinkRetCode_t | 参考 JoylinkRetCode_E 定义 |

**附加说明**

使用该API时，in_post中的设备标识为**子设备**的设备标识。

* * * *





###  5.4.  SDK API 调用顺序及流程

使用SDK进行设备接入，SDK API的调用顺序，可按照下面的示例代码进行：

```c
int main()
{
    /***********************************************************************
     * 测试函数
     ***********************************************************************/
    joylink_sdk_test();
    
    /***********************************************************************
     * joylink sdk 配置参数
     ***********************************************************************/
    joylink_sdk_config_t *config = joylink_sdk_config_create();
    if (config == NULL) {
        log_error("joylink sdk config create failed");
        return -1;
    }
    jl_platform_memset(config, 0, sizeof(joylink_sdk_config_t));
    
    // 主机地址和端口号
    config->hostname = SDK_CONFIG_HOST_NAME;
    config->port = SDK_CONFIG_HOST_PORT;

    // 设备id，用户名和密码
    config->deviceId = SDK_CONFIG_DEVICE_ID;
    config->username = SDK_CONFIG_DEVICE_ID;

    // TLS 证书
    config->cafile = SDK_CONFIG_CA_PATH;
    config->cert = SDK_CONFIG_CERT_PATH;
    config->key = SDK_CONFIG_KEY_PATH;

    // 验证主机名
    config->insecure = true;

    /***********************************************************************
     * joylink sdk 创建
     ***********************************************************************/
    struct joylink_sdk_t *joy_sdk = joylink_sdk_create(config);

    /***********************************************************************
     * joylink sdk 初始化
     ***********************************************************************/
    joylink_sdk_initialise(joy_sdk);

    /***********************************************************************
     * joylink sdk 设置回调函数
     ***********************************************************************/
    // 设置 连接成功回调函数
    joylink_sdk_set_dev_cb_connect(joy_sdk, device_callback_connect);

    // 设置 连接断开回调函数
    joylink_sdk_set_dev_cb_disconnect(joy_sdk, device_callback_disconnect);

    // 设置 物模型上报响应回调函数
    joylink_sdk_set_dev_cb_thmd_post_res(joy_sdk, device_callback_thingmodel_post_response);

    // 设置 属性设置回调函数
    joylink_sdk_set_dev_cb_prop_set(joy_sdk, device_callback_property_set);

    // 设置 属性获取回调函数
    joylink_sdk_set_dev_cb_prop_get(joy_sdk, device_callback_property_get);

    // 设置 方法调用回调函数
    joylink_sdk_set_dev_cb_func_call(joy_sdk, device_callback_function_call);

    // 设置 自动注册响应回调函数
    joylink_sdk_set_dev_cb_reg_res(joy_sdk, device_callback_register_response);

    // 设置 ntp请求回调函数
    joylink_sdk_set_dev_ntp_req_res(joy_sdk, device_callback_ntp_response);
    /***********************************************************************
     * 设备 初始化 可根据实际情况进行
     ***********************************************************************/
    device_initialise(joy_sdk);

    /***********************************************************************
     * joylink sdk 连接京东云
     ***********************************************************************/
    joylink_sdk_connect(joy_sdk);
    
    /***********************************************************************
     * joylink sdk 进入主循环
     ***********************************************************************/
    joylink_sdk_main_loop(joy_sdk);

    /***********************************************************************
     * joylink sdk 销毁 释放资源
     ***********************************************************************/
    joylink_sdk_destory(joy_sdk);

    /***********************************************************************
     * joylink sdk 配置 销毁 释放资源
     ***********************************************************************/
    jl_platform_memset(config, 0, sizeof(joylink_sdk_config_t));
    joylink_sdk_config_destory(config);

    return 0;
}
```



###  5.5.  用户实现部分说明
参考上面的示例代码，需要用户实现的部分详细说明如下：

####  5.5.1.  SDK 配置参数
在进行设备接入之前，需要您在物联平台完成设备的创建及注册相关操作流程，并且获取了以下参数：

> **hostname**：京东云主机地址；
> **post**：京东云主机端口号；
> **deviceId**：设备标识，用户名（username）填设备标识，密码（password）为空；
> **ca**：TLS根证书，路径填入ca；
> **cert**：cert证书，路径填入cert；
> **key**：key证书，路径填入key；


####  5.5.2.   设备回调函数
SDK从物联平台接收到消息或者响应，通过回调函数的形式交由设备处理。如果设备对某些功能不支持也可以选择不实现对应的回调函数。



##### 5.5.2.1.  连接成功回调函数

SDK与物联平台连接成功后，会调用该函数。这里的连接不仅仅是初次连接，也包括因故断开连接后的重新连接。

用户定义实现好回调函数后，可以调用
```c
        joylink_sdk_set_dev_cb_connect
```
API 函数来完成回调函数的设置。下面给出一个回调函数定义的示例代码以供参考：

```c
int32_t device_callback_connect(struct joylink_sdk_t *sdk)
{
    // 用户可根据自身情况进行逻辑处理

    device_demo.is_connected = 1;

    return JOYLINK_RET_SUCCESS;
}
```

用户可以在回调函数中更改当前的连接状态，物模型上报，属性上报，以及其它一些需要处理的逻辑。



##### 5.5.2.2.  连接断开回调函数

SDK与物联平台的连接断开时，不管是主动断开还是被动断开，都会调用该函数。

用户定义实现好回调函数后，可以调用
```c
        joylink_sdk_set_dev_cb_disconnect
```
API 函数来完成回调函数的设置。下面给出一个回调函数定义的示例代码以供参考：

```c
int32_t device_callback_disconnect(struct joylink_sdk_t *sdk)
{
    // 用户可根据自身情况进行逻辑处理
    device_demo.is_connected = 0;

    return JOYLINK_RET_SUCCESS;
}
```
用户可以在回调函数中进行连接状态的更新，以及其它一些需要进行的逻辑处理。



##### 5.5.2.3.  物模型上报响应回调函数

用户主动发布了物模型信息上报之后，物联平台收到后会给出相应的响应，用户可以通过设置回调函数来查看确认物模型上报的结果。

用户定义实现好回调函数后，可以调用
```c
        joylink_sdk_set_dev_cb_thmd_post_res
```
API 函数来完成回调函数的设置。下面给出一个回调函数定义的示例代码以供参考：
```c
int32_t device_callback_thingmodel_post_response(
                      struct joylink_sdk_t *sdk,
                      char *obj_name,
                      char *service_key,
                      JoyProtoThingModelPostRes_t *in_res)
{
    
    /* 示例代码
     * if (in_res->code != 200) {log_error("failed")};
     */

    return JOYLINK_RET_SUCCESS;
}
```
用户可根据返回码，来确定物模型上报的结果，及后续的一些逻辑。

**附加说明**

> [1] 对象空间(obj_name)的取值，SDK目前仅支持 {”device“}；
> 
> [2] 服务实例名(service_key)的取值，对于物模型上报响应来说，目前为NULL，开发者可以暂时不用进行逻辑判断；



##### 5.5.2.4.  属性设置回调函数

SDK接收到物联平台下发的属性设置消息时，会调用该函数。

用户定义实现好回调函数后，可以调用
```c
        joylink_sdk_set_dev_cb_prop_set
```
API 函数来完成回调函数的设置。下面给出一个回调函数定义的示例代码以供参考：
```c
int32_t device_callback_property_set(
                      struct joylink_sdk_t *sdk,
                      char *obj_name,
                      char *service_key,
                      JoyProtoPropSet_t *in_set)
{
    // 对设备进行属性更改


    // 发送 属性设置响应示例代码
    JoyProtoPropSetRes_t out_res;
    out_res.deviceId = in_set->deviceId;
    out_res.messageId = in_set->messageId;
    /*
     * out_res.properties[0]->value = joylink_proto_keyvalue_pack_int32(200);
     * 注意内存释放问题
     */
    joylink_sdk_dev_prop_set_response(sdk, obj_name, service_key, &out_res);

    return JOYLINK_RET_SUCCESS;
}
```
用户需要在回调函数中，完成对设备属性的修改，并发布属性设置响应消息。

**附加说明**
> [1] 对象空间(obj_name)的取值，SDK目前仅支持 {”device“}；
>
> [2] 服务实例名(service_key)，用户需要进行区分判断。如果为NULL，则表示是设备本身的属性；如果是“connection_agent”， 则表明要对子设备的属性进行设置，用户需要根据设备标识来区分是哪个子设备；
>
> [3] 属性值说明 in_set->properties[0]->value 为json字符串，sdk 提供了与bool，int32_t，double和string转换的接口，其它类型的需要用户自行解析。
>
> 比如要获取int32_t值：
> ```c
> joylink_proto_keyvalue_parse_int32(char *in_value, int32_t *out_val);
> ```
> 要打包in32_t值：
> ```c
> char *joylink_proto_keyvalue_pack_int32(int32_t in_int32);
> ```
>
> [4] 需要对版本号（version）进行校验，只有请求的版本号大于当前保存的版本号才能执行属性设置操作，如果版本号验证通过，需要将设备保存的版本号更新成最新请求的版本号；
>
> [5] 处理完后需要发送响应；
>
> [6] 对于网关代理设备，也可以通过 joylink_sdk_set_sub_dev_cb_prop_set 直接注册子设备的属性设置回调；



##### 5.5.2.5.  属性获取回调函数

SDK接收到物联平台下发的属性获取消息时，会调用该函数。

用户定义实现好回调函数后，可以调用
```c
        joylink_sdk_set_dev_cb_prop_get
```
API 函数来完成回调函数的设置。下面给出一个回调函数定义的示例代码以供参考：
```c
int32_t device_callback_property_get(
                      struct joylink_sdk_t *sdk,
                      char *obj_name,
                      char *service_key,
                      JoyProtoPropGet_t *in_get)
{
    // 服务实例名，需要开发这进行判断处理
    
    // 处理流程及属性值(properties[0]->value) 参见属性设置的说明

    // 发送 属性设置响应示例代码
    JoyProtoPropGetRes_t out_res;
    out_res.deviceId = in_get->deviceId;
    out_res.messageId = in_get->messageId;
    /*
     * out_res.properties[0]->value = joylink_proto_keyvalue_pack_int32(200);
     * 注意内存释放问题
     */
    joylink_sdk_dev_prop_get_response(sdk, obj_name, service_key, &out_res);

    return JOYLINK_RET_SUCCESS;
}
```
**附加说明**
> [1] 对象空间(obj_name)的取值，SDK目前仅支持 {”device“}；
>
> [2] 服务实例名(service_key)，用户需要进行区分判断。如果为NULL，则表示是设备本身的属性；如果是“connection_agent”， 则表明要获取子设备的属性，用户需要根据设备标识来区分是哪个子设备；
>
> [3] 如果获取属性的个数为0，则要返回所有的属性；
>
> [4] 对于网关代理设备，可以通过 joylink_sdk_set_sub_dev_cb_prop_get 直接注册子设备的属性获取回调函数；



##### 5.5.2.6.  方法调用回调函数

SDK接收到物联平台下发的方法调用消息时，会调用该函数。

用户定义实现好回调函数后，可以调用
```c
        joylink_sdk_set_dev_cb_func_call
```
API 函数来完成回调函数的设置。下面给出一个回调函数定义的示例代码以供参考：
```c
int32_t device_callback_function_call(
                      struct joylink_sdk_t *sdk,
                      char *obj_name,
                      char *service_key,
                      JoyProtoFuncCall_t *in_call)
{
   
    // 方法调用入参及出参 参数值(in[0]->value) 与属性值相同为 json 字符串

    // ota
    if (!jl_platform_strcmp(service_key, JOYLINK_PROTO_SERVICE_OTA)) {
        return device_ota_process_func_call(sdk, obj_name, in_call);
    }

    // 发送 属性设置响应示例代码
    JoyProtoFuncCallRes_t out_res;
    out_res.deviceId = in_call->deviceId;
    out_res.messageId = in_call->messageId;
    /*
     * out_res.functions[0]->out[0]->value = joylink_proto_keyvalue_pack_int32(200);
     * 注意内存释放问题
     */
    joylink_sdk_dev_func_call_response(sdk, obj_name, service_key, &out_res);

    return JOYLINK_RET_SUCCESS;
}
```


**附加说明**

> [1] 对象空间(obj_name)的取值，SDK目前仅支持 {”device“}；
>
> [2] 目前默认每次只有一个函数调用 即 in_call->func_num == 1；
>
> [3] 如果时网关设备，并且需要使用物联平台下发的子设备的信息，那么当服务实例名（service_key） 为 ”connection_agent“ 时，需要根据实际的需求处理服务内置的四个方法调用，方法调用的实例名如下：
>
> > 添加子设备：”connection_agent.add-devices“；
> > 删除子设备：“connection_agent.delete-devices”；
> > 更新子设备：“connection_agent.update-devices”；
> > 查询子设备：“connection_agent.get-devices”；
>
> [4] 如果是网关设备时，需要根据消息里的设备标识来判断是调用的哪个子设备的方法。
>
> [5] 如果服务实例名（service_key） 为 ”ota“ 时，需要根据实际需求处理ota服务内置的4个方法调用：
>
> > ota指令：”ota.ota“；
> > 取消任务：“ota.cancel-task”；
> > 执行任务：“ota.exec-task”；
> > 查询状态：“ota.query-ota-state”；
>
> [5] 方法调用完成后，需要给出响应。
>
> [6] 对于网关设备，可以通过 joylink_sdk_set_agent_cb_func_call 直接注册连接代理服务的方法调用，同样可以通过 joylink_sdk_set_sub_dev_cb_func_call 来直接注册子设备的方法调用；



##### 5.5.2.7.  自动注册响应回调函数
待续

##### 5.5.2.7.  设备ntp授时响应回调函数
用户主动发出ntp请求后，当SDK收到ntp授时的响应时，会调用该函数。
用户定义实现好回调函数后，可以调用
```c
        joylink_sdk_set_dev_ntp_req_res
```
API 函数来完成回调函数的设置。下面给出一个回调函数定义的示例代码以供参考：
```c
int32_t device_callback_ntp_response(struct joylink_sdk_t *sdk, jl_time_stamp_t *set_time)
{
    // TODO 如果有需要，进行时钟同步

    log_info("NTP Time Needed to be Set: %d.%d", set_time->second, set_time->ms);

    return JOYLINK_RET_SUCCESS;
}
```


####  5.5.3.  设备主动发起消息
有些消息需要设备主动进行上报，现详细说明如下：
#####  5.5.3.1.  物模型上报
SDK与物联平台连接成功后，需要进行物模型上报。示例代码如下：
```c
    JoyProtoThingModelPost_t post;
    jl_platform_memset(&post, 0, sizeof(JoyProtoThingModelPost_t));
    post.deviceId = "my_device_id";
    post.messageId = "my_message_id";
    post.timestamp = jl_platform_time(NULL);
    post.thing_model.id = "my_thingmodel_id";
    post.thing_model.version = "my_thingmodel_version";
    joylink_sdk_dev_thing_model_post(sdk, JOYLINK_PROTO_OBJ_NAME_DEV, NULL, &post);
```



#####  5.5.3.2.  属性上报

在以下几种情况下，需要进行属性上报：
> [1] SDK与物联平台连接成功时， 需要上报设备当前属性值，包括直连设备，网关设备（如果定义了属性）;
> 
> [2] 对被代理的子设备，当网关设备探测到子设备上线时，需要代理子设备上报当前的属性；
> 
> [3] 当设备的属性有变化时，也需要主动上报变化后的属性；

属性上报的参考示例代码如下：
```c
    JoyProtoPropPost_t prop_post;
    jl_platform_memset(&prop_post, 0, sizeof(JoyProtoPropPost_t));
    // prop_post.version = current_version + 1;
    joylink_sdk_dev_prop_post(sdk, JOYLINK_PROTO_OBJ_NAME_DEV, NULL, &prop_post);
```

**附加说明**
> [1]  如果是直连设备或者网关设备的属性，prop_post.deviceId 填入直连设备或者网关设备自身的设备标识，API 函数入参服务实例名填NULL；
>
> [2]  如果是代理子设备的属性，prop_post.deviceId 填入子设备的设备标识，API 函数入参服务实例名填 “connection_agent”；
>
> [3] 关于version版本，如果上报的属性中含有**可写**属性，version值需要加 1；
>
> [4] 对于网关设备，可以调用 joylink_sdk_sub_dev_prop_post 来代理子设备进行属性上报；



#####  5.5.3.3.  直连设备上下线上报

对于直连设备或者网关设备本身，以下情况需要主动上报上线消息：
> [1] SDK与物联平台连接成功;

对于上线消息发送，示例代码如下：
```c
    JoyProtoEvtOnlineStatus_t status;
    jl_platform_memset(&status, 0, sizeof(JoyProtoEvtOnlineStatus_t));
    status.is_onlue = true;
    // 设备标识，消息Id等参数，填入
    joylink_sdk_dev_evt_online_status_post(sdk, JOYLINK_PROTO_OBJ_NAME_DEV, NULL, &status);
```



以下情况需要主动上报下线消息：

> [1] 当设备由于主动的原因要离线时，如OTA重启等情况，也可以主动地发送下线消息;

对于下线消息发送，示例代码如下：
```c
    JoyProtoEvtOnlineStatus_t status;
    jl_platform_memset(&status, 0, sizeof(JoyProtoEvtOnlineStatus_t));
    status.is_onlue = false;
    // 设备标识，消息Id等参数，填入
    joylink_sdk_dev_evt_online_status_post(sdk, JOYLINK_PROTO_OBJ_NAME_DEV, NULL, &status);
```
**附加说明**
> [1]  只有直连设备和网关设备自身的上线消息通过这个API进行发送， API 函数入参服务实例名填NULL；



#####  5.5.3.4.  事件上报
当直连设备、网关设备，或者子设备检测到有事件被触发时，需要进行事件上报。示例代码如下：
```c
    JoyProtoEvtPost_t evt_post;
    jl_platform_memset(&evt_post, 0, sizeof(JoyProtoEvtPost_t));
    joylink_sdk_dev_evt_post(sdk, JOYLINK_PROTO_OBJ_NAME_DEV, NULL, &evt_post);
```
**附加说明**
> [1]  如果是直连设备或者网关设备本身事件上报，evt_post.deviceId 为直连设备或者网关设备自身的设备标识，并且API 函数服务实例名填NULL；
>
> [2]  如果是网关设备代理子设备进行事件上报，evt_post.deviceId 为子设备的设备标识，并且API 函数服务实例名填 “connection_agent"；
>
> [3] 对于网关设备，也可以通过 joylink_sdk_sub_dev_evt_post 代理子设备进行事件上报；



#####  5.5.3.5.  子设备上下线上报

当网关设备检测到子设备上线或者下线时，需要代理子设备进行上下线消息上报；子设备上下线上报是通过connection_agent 服务内置的事件实现的。示例代码参见事件上报；

**附加说明**
> [1]  消息报文 evt_post.deviceId 填写网关设备的设备标识；
>
> [2] API 函数入参服务实例名填 “connection_agent”;
>
> [3] 子设备上下线的事件实例名为：
>> 上线事件：“connection_agent.online”；
>> 下线事件：“connection_agent.offline”；
>
> [4] 事件参数 devices 的值是子设备信息的数组，即数据类型为数组；



如果要对单个子设备的上线下线状态进行上报，可以使用API：

```c
// 代理子设备上报 上线状态
joylink_sdk_sub_dev_online_post

// 代理子设备上报 下线状态
joylink_sdk_sub_dev_offline_post
```



如果要批量进行多个设备多种状态的上报，开发者需要自行根据事件上报的流程来实现。



#####  5.5.3.6.  自动注册请求

待续

#####  5.5.3.7.  设备ntp授时请求
当设备需要校正本地时间的时候，可以调用
```c
joylink_sdk_dev_ntp_request
```
来请求ntp授时；


####  5.5.4.  协议约定的一些说明

本节对协议的一些约定进行总结说明，这些约定在之前的章节中可能已经出现过，这里只是整体上进行梳理，以期加深开发者对协议约定的印象，强烈建议用户仔细认真阅读；



#####  5.5.4.1.  属性版本号
> [1] 对于设备，不管是直连设备，网关设备还是被代理的子设备，只要有属性，就需要维护版本号；

> [2] 属性版本号范围及大小约定：版本号范围是 0~65535，并且约定 0  > 65535 ;

> [3] 设备接收到属性设置请求时，设备需要对版本号进行判断，只有请求消息中的版本号大于当前设备维护的版本号，才能对设备进行属性设置，并且属性设置完成后要将设备维护的版本号更新为消息请求的版本号，否则可以拒绝该请求；（这条约定主要考虑到可能会有不同的控制方同时发起对设备的操作，并且物联平台不能保证到达的先后顺序）

> [4]  设备主动进行属性上报时，如果上报的属性其中含有可写属性，需要将设备维护的版本号加1，再进行上报；


> [5] 断电重启或者断网重连时：请根据设备的实际情况  ___谨慎___  处理，如果设备有特有的重启或者重连后的安全状态，在进行属性上报的时候请根据京东云版本号的处理逻辑 ___谨慎___  操作版本号；



#####  5.5.4.2.  消息中的设备标识

> [1] 直连设备，消息中的设备标识填设备本身的设备标识；

> [2] 网关设备，如果消息是网关设备本身的，消息中的设备标识填网关设备本身的标识；如果消息是子设备的，消息标识填子设备的设备标识；



#####  5.5.4.3.  connection_agent 服务 

> [1] 服务内置了子设备相关的四种方法调用，如果网关设备需要使用从物联平台下发的子设备信息，那么用户需要自行处理；如果不需要，可以忽略；

> [2] 代理子设备进行上线，下线状态上报，需要使用服务内置的事件来实现； 



#####  5.5.4.4.  方法调用

> [1] 用户自定义的方法调用执行的结果，比如成功或者失败，要在消息体中的code中体现，具体code代表的含义，请参见与物联平台的约定；

> [2] 内置的方法调用，方法调用出入参已定义，用户只需要根据定义实现即可；



#####  5.5.4.5.  消息标识

> [1] 所有的消息体中需要包括消息标识（messageId）字段；

> [2] 响应消息体中消息标识（messageId）必须与设置，请求，调用中的保持一致;



#####  5.5.4.6.  设备ntp授时

> [1] 这是一个可选服务，如果设备有其它的途径进行时钟同步，可以不选用；

> [2] 该服务的时间标准是物联管理平台的时钟；;



####  5.5.5.  直连设备开发总结说明

如果接入的是直连设备，需要实现的功能及相关说明总结如下：
> [1] 直连设备物模型上报，上下线上报；
>
> [2] 属性上报，包括连接成功后的上报和属性变化时的上报；
>
> [3] 属性设置功能，此项跟属性的定义有关，如果所有的属性都是可读属性，此功能可不必实现；
>
> [4] 属性获取功能；
>
> [5] 方法调用功能，如果设备没有定义方法，此功能可不必实现；
>
> [6] 事件上报功能，如果设备没有定义事件，此功能可不必实现；
>
> [7] 注册的相应的回调函数中，相关逻辑实现，不需要考虑服务实例名是 “connection_agent” 的情况；



####  5.5.6.  网关设备开发总结说明

如果接入的是网关设备，需要实现的功能及相关说明总结如下：
> [1] 网关设备本身实际上是一个直连设备，因此需要根据直连设备的相关说明实现作为直连设备的这部分功能；**（消息里面的设备标识为网关设备本身的设备标识，服务实例名为NULL）**；
>
> [2] connection_agent 服务内置的关于子设备的4个方法，即子设备的增加、删除、更新和查询，需要根据实际使用场景来决定要不要实现；
>
> [3] 网关设备需要使用 connection_agent 服务内置的 子设备上线、下线事件代理子设备进行上线，下线状态的上报；**（消息里面的设备标识为网关设备本身的设备标识，服务实例名为NULL）**
>
> [4] 代理子设备进行属性上报，属性设置响应，属性获取响应，方法调用响应，事件上报时， **消息里面的设备标识为子设备的设备标识，服务实例名为 “connection_agent”**；
>
> [5] 对于网关设备的开发，开发者可以根据说明使用低级别的API实现，也可以通过网关相关的API来实现。

* * *



###  5.6.  OTA开发说明

设备的OTA功能是通过内置的ota服务来实现；ota服务有内置的属性、方法和事件。因此如若进行ota功能的开发，需要先进行属性获取和方法调用回调函数的设置。

####  5.6.1. ota方法调用处理
在方法调用回调函数中，如果服务实例名为"ota"， 需要处理ota服务的内置方法（开发者也可以直接调用 joylink_sdk_set_dev_ota_cb_func_call 直接注册OTA方法调用的回调函数），OTA核心处理流程逻辑相同；
```c
int32_t device_callback_function_call(
                      struct joylink_sdk_t *sdk,
                      char *obj_name,
                      char *service_key,
                      JoyProtoFuncCall_t *in_call)
{
   
    // 方法调用入参及出参 参数值(in[0]->value) 与属性值相同为 json 字符串

    // ota
    if (!jl_platform_strcmp(service_key, JOYLINK_PROTO_SERVICE_OTA)) {
        return device_ota_process_func_call(sdk, obj_name, in_call);
    }

    // 发送 方法调用响应示例代码
    JoyProtoFuncCallRes_t out_res;
    out_res.deviceId = in_call->deviceId;
    out_res.messageId = in_call->messageId;
    /*
     * out_res.functions[0]->out[0]->value = joylink_proto_keyvalue_pack_int32(200);
     * 注意内存释放问题
     */
    joylink_sdk_dev_func_call_response(sdk, obj_name, service_key, &out_res);

    return JOYLINK_RET_SUCCESS;
}
```

在example/joylink_device_ota.c 中给出了ota方法处理的示例代码如下:
```c
int32_t device_ota_process_func_call(struct joylink_sdk_t *sdk, char *obj_name, JoyProtoFuncCall_t *in_call)
{
    // 根据方法调用的实例名判断
    if (!jl_platform_strcmp(in_call->functions[0]->key, JOYLINK_PROTO_OTA_FUNC_OTA)) {
        return device_ota_process_func_ota(sdk, obj_name, in_call);
    }
    else if (!jl_platform_strcmp(in_call->functions[0]->key, JOYLINK_PROTO_OTA_FUNC_QUERY_STATE)) {
        return device_ota_process_func_query_state(sdk, obj_name, in_call);
    }
    else {
        log_info("Unsupport OTA Function Call: %s", in_call->functions[0]->key);
    }

    return JOYLINK_RET_SUCCESS;
}
```
**附加说明**
> [1] 需要根据方法调用中具体的方法实例名来判断具体需要执行的是哪种方法调用；
>
> [2] 目前仅支持{"ota.ota", "ota. query-ota-state"} 两种方法调用；



#####  5.6.1.1.   ota.ota 方法调用

ota.ota 方法主要作用是物联平台向设备发送ota升级任务，开发者需要根据方法入参和设备实际的情况进行ota相关操作。对ota.ota方法的支持的入参说明如下：
```c
// 任务标识
#define D_OTA_FUNC_ARG_TASK_ID              JOYLINK_PROTO_OTA_TASK_ID

// OTA 包名
#define D_OTA_FUNC_ARG_PKG_NAME             JOYLINK_PROTO_OTA_PKG_NAME

// 升级包版本名称
#define D_OTA_FUNC_ARG_NAME                 ("name")

// 升级包版本号
#define D_OTA_FUNC_ARG_VERSION              JOYLINK_PROTO_OTA_VERSION

// 在对子设备升级时，此为子设备id
#define D_OTA_FUNC_ARG_OBJECT_ID            JOYLINK_PROTO_OTA_OBJECT_ID

// 升级对象类型 {"app", "firmware", "framework", "os"} 目前仅支持 firmware
#define D_OTA_FUNC_ARG_OBJECT_TYPE          ("object-type")

// 全量包获取网址
#define D_OTA_FUNC_ARG_URL                  ("url")

// 升级模式 {"0":"立即安装", "1":"用户确认后下载安装", "2":"用户确认后安装","3":"定时安装", "4":"闲时安装"}
#define D_OTA_FUNC_ARG_INSTALL_MODE         ("install-mode")

// 升级执行时间
#define D_OTA_FUNC_ARG_TIME                 ("time")

// 升级全量包大小，以字节为单位
#define D_OTA_FUNC_ARG_SIZE                 ("size")

// 用户自定义的辅助信息
#define D_OTA_FUNC_ARG_CUSTOMIZED           ("customized")

// 升级包签名算法 {"1":"SHA256", "2":"SHA256WithRSA","3":"SM3","4":"SM3WithSM2","5":"MD5"}
#define D_OTA_FUNC_ARG_SIG_ALG				("signature-algorithm")
```
**附加说明**
> [1] 任务标识 参数，需要开发者进行一些消息保存，在后续进行ota状态变化事件上报，以及任务取消、执行和查询时候会用到该参数；
> [2] 其它一些参数，需要开发者根据实际情况进行甄别是否需要保存或者维护；
>
> [3] 需要判断方法调用的版本号与当前版本号的大小，防止降级；
>
> [4] 方法调用无出参；

下面给出示例代码如下：
```c
int32_t device_ota_process_func_ota(struct joylink_sdk_t *sdk, char *obj_name, JoyProtoFuncCall_t *in_call)
{
    log_info("OTA Function Key:%s", in_call->functions[0]->key);
    int32_t ii;
    char *tmp_char = NULL;
    for (ii = 0; ii < in_call->functions[0]->in_num; ii++) {
        jl_platform_printf("\t%s:\t\t%s\r\n", in_call->functions[0]->in[ii]->key, in_call->functions[0]->in[ii]->value);

        // 暂时保存需要的信息
        if (!jl_platform_strcmp(in_call->functions[0]->in[ii]->key, D_OTA_FUNC_ARG_URL)) {
            tmp_char = joylink_proto_keyvalue_parse_string(in_call->functions[0]->in[ii]->value);
            if (tmp_char != NULL) {
                jl_platform_strcpy(ota_url, tmp_char);
                jl_platform_free(tmp_char);
                tmp_char = NULL;
            }
        }
        // 判断安装方式
        else if (!jl_platform_strcmp(in_call->functions[0]->in[ii]->key, D_OTA_FUNC_ARG_INSTALL_MODE)) {
            tmp_char = joylink_proto_keyvalue_parse_string(in_call->functions[0]->in[ii]->value);
            if (tmp_char != NULL) {
                // 立即安装
                if (!jl_platform_strcmp(tmp_char, D_OTA_INSTALL_MODE_IMM)) {
                }
                // 其它形式，暂不支持
                else {
                }
                jl_platform_free(tmp_char);
                tmp_char = NULL;
            }
        }
        // 保存任务标识
        else if (!jl_platform_strcmp(in_call->functions[0]->in[ii]->key, D_OTA_FUNC_ARG_TASK_ID)) {
            tmp_char = joylink_proto_keyvalue_parse_string(in_call->functions[0]->in[ii]->value);
            if (tmp_char != NULL) {
                jl_platform_strcpy(ota_task_id, tmp_char);
                jl_platform_free(tmp_char);
                tmp_char = NULL;
            }
        }
        else {}
    }

    // TODO OTA 任务线程
    // 判断调用参数版本号与当前版本号的大小，防止降级
    // 需要根据实际的升级类型选择选择执行逻辑
    // 这里给出了立即执行的例子

    jl_thread_t task_id;
    task_id.thread_task = (threadtask) device_ota_task_thread;
	task_id.stackSize = 0x4000;
    task_id.priority = JL_THREAD_PRI_DEFAULT;
    task_id.parameter = (void *) sdk;
    jl_platform_thread_start(&task_id);


    // 方法响应
    JoyProtoFuncCallRes_t call_res;
    jl_platform_memset(&call_res, 0, sizeof(JoyProtoFuncCallRes_t));
    call_res.deviceId = in_call->deviceId;
    call_res.messageId = in_call->messageId;
    call_res.timestamp = jl_platform_time(NULL);
    call_res.code = 200;
    call_res.func_num = 1;
    call_res.functions = (JoyProtoFuncCallResFunc_t **) jl_platform_malloc(
                      call_res.func_num * sizeof(JoyProtoFuncCallResFunc_t *));
    jl_platform_memset(call_res.functions, 0, call_res.func_num * sizeof(JoyProtoFuncCallResFunc_t *));
    call_res.functions[0] = joylink_proto_ota_create_func_res(in_call->functions[0]->key);
    
    joylink_sdk_dev_func_call_response(sdk, obj_name, JOYLINK_PROTO_SERVICE_OTA, &call_res);

    // 释放内存
    call_res.deviceId = NULL;
    call_res.messageId = NULL;
    joylink_proto_free_func_call_res(&call_res);
    return JOYLINK_RET_SUCCESS;
}
```



#####  5.6.1.2.   ota.cancel-task 方法调用

ota.cancel-task 方法主要作用是物联平台向设备发送取消ota升级指令，开发者需要根据方法入参中的任务标识，来取消掉相应的正在进行或者还未进行的升级任务。方法的入参说明如下：

```c
// 任务标识
#define D_OTA_FUNC_ARG_TASK_ID              JOYLINK_PROTO_OTA_TASK_ID

// 在对子设备升级时，此为子设备id
#define D_OTA_FUNC_ARG_OBJECT_ID            JOYLINK_PROTO_OTA_OBJECT_ID
```

处理的示例代码如下

```c
int32_t device_ota_process_func_cancel_task(struct joylink_sdk_t *sdk, char *obj_name, JoyProtoFuncCall_t *in_call)
{
    log_info("OTA Function Key:%s", in_call->functions[0]->key);
    int32_t ii;
    for (ii = 0; ii < in_call->functions[0]->in_num; ii++) {
        jl_platform_printf("\t%s:\t\t%s\r\n", in_call->functions[0]->in[ii]->key, in_call->functions[0]->in[ii]->value);
    }

    // TODO 用户需要根据实际的情况来执行取消任务
    // 取消OTA
    ota_task_cancel = 1;

    // 方法响应
    JoyProtoFuncCallRes_t call_res;
    jl_platform_memset(&call_res, 0, sizeof(JoyProtoFuncCallRes_t));
    call_res.deviceId = in_call->deviceId;
    call_res.messageId = in_call->messageId;
    call_res.timestamp = jl_platform_time(NULL);
    call_res.code = 200;
    call_res.func_num = 1;
    call_res.functions = (JoyProtoFuncCallResFunc_t **) jl_platform_malloc(
                      call_res.func_num * sizeof(JoyProtoFuncCallResFunc_t *));
    jl_platform_memset(call_res.functions, 0, call_res.func_num * sizeof(JoyProtoFuncCallResFunc_t *));
    call_res.functions[0] = joylink_proto_ota_create_func_res(in_call->functions[0]->key);

    joylink_sdk_dev_func_call_response(sdk, obj_name, JOYLINK_PROTO_SERVICE_OTA, &call_res);

    // 释放内存
    call_res.deviceId = NULL;
    call_res.messageId = NULL;
    joylink_proto_free_func_call_res(&call_res);

    return JOYLINK_RET_SUCCESS;
}
```



#####  5.6.1.3.  ota.exec-task 方法调用

暂不支持



#####  5.6.1.4.  ota. query-ota-state 方法调用

ota. query-ota-state 作用是物联平台向设备查询指定ota任务的当前状态，方法的入参是：
```c
// 任务标识
#define D_OTA_FUNC_ARG_TASK_ID              JOYLINK_PROTO_OTA_TASK_ID

// 在对子设备升级时，此为子设备id
#define D_OTA_FUNC_ARG_OBJECT_ID            JOYLINK_PROTO_OTA_OBJECT_ID
```
**附加说明**
> [1] 方法有三个出参，任务标识，当前状态，和当前进度，sdk对出参进行了API封装；

方法处理及响应出参API封装使用示例代码如下：
```c
int32_t device_ota_process_func_query_state(struct joylink_sdk_t *sdk, char *obj_name, JoyProtoFuncCall_t *in_call)
{
    log_info("OTA Function Key:%s", in_call->functions[0]->key);
    int32_t ii;
    char *tmp_string = NULL;
    for (ii = 0; ii < in_call->functions[0]->in_num; ii++) {
        jl_platform_printf("\t%s:\t\t%s\r\n", in_call->functions[0]->in[ii]->key, in_call->functions[0]->in[ii]->value);

        // 获取任务标识
        if (!jl_platform_strcmp(in_call->functions[0]->in[ii]->key, D_OTA_FUNC_ARG_TASK_ID)) {
            tmp_string = joylink_proto_keyvalue_parse_string(in_call->functions[0]->in[ii]->value);
        }
    }

    // TODO
    // 执行状态查询

    // 方法响应
    JoyProtoFuncCallRes_t call_res;
    jl_platform_memset(&call_res, 0, sizeof(JoyProtoFuncCallRes_t));
    call_res.deviceId = in_call->deviceId;
    call_res.messageId = in_call->messageId;
    call_res.timestamp = jl_platform_time(NULL);
    call_res.code = 200;
    call_res.func_num = 1;
    call_res.functions = (JoyProtoFuncCallResFunc_t **) jl_platform_malloc(
                      call_res.func_num * sizeof(JoyProtoFuncCallResFunc_t *));
    jl_platform_memset(call_res.functions, 0, call_res.func_num * sizeof(JoyProtoFuncCallResFunc_t *));
    call_res.functions[0] = joylink_proto_ota_create_func_query_res(in_call->functions[0]->key,
                                                                    tmp_string, JOY_OTA_ST_INSTALLING, 55);

    joylink_sdk_dev_func_call_response(sdk, obj_name, JOYLINK_PROTO_SERVICE_OTA, &call_res);

    // 释放内存
    call_res.deviceId = NULL;
    call_res.messageId = NULL;
    joylink_proto_free_func_call_res(&call_res);

    if (tmp_string != NULL) {
        jl_platform_free(tmp_string);
    }

    return JOYLINK_RET_SUCCESS;
}
```


####  5.6.2. ota事件上报

ota服务有内置了两个事件，分别说明如下：
> [1] 状态变化事件："ota.ota-state-changed"
> [2] 版本号变化事件："ota.ota-version-changed"



#####  5.6.2.1.  ota.ota-state-changed 事件

设备执行ota升级过程中，如果状态出现了变化，或者过程中有错误发生了，可通过该事件进行上报。API中参数的详细说明请查看前面的章节，这里给出使用示例代码：
```c
    joylink_sdk_dev_ota_state_changed_post(sdk, "device", ota_task_id, JOY_OTA_ST_INSTALL_DONE, JOY_OTA_ERROR_NONE, 100);
```
**附件说明**
> [1] 上述入参代表的意思是 执行完毕，没有错误，进度100%



#####  5.6.2.2.  ota.ota-version-changed 事件

以下两种情况需要通过该事件将版本号上报至物联管理平台。
> [1] 设备初次连接到物联管理平台后，上报当前的版本号；
> [2] 进行OTA升级完成后，版本号发生改时；

API中参数的详细说明请查看前面的章节，这里给出使用示例代码：
```c
joylink_sdk_dev_ota_version_changed_post(sdk, "device", NULL, "v1.0.1");
```



####  5.6.3. ota属性上报

OTA服务有4个内置的属性，进行属性上报时，参考如下代码：

```c
static int32_t device_ota_properties_post(struct joylink_sdk_t *sdk)
{
    // 连接成功 ota 属性上报
    JoyProtoPropPost_t prop_post;
    jl_platform_memset(&prop_post, 0, sizeof(JoyProtoPropPost_t));
    prop_post.deviceId = SDK_CONFIG_DEVICE_ID;
    prop_post.timestamp = jl_platform_time(NULL);
    prop_post.version = 0;
    prop_post.messageId = joylink_sdk_get_messageId(NULL);
    prop_post.properties = device_ota_get_properties(&prop_post.prop_num);

    joylink_sdk_dev_prop_post(sdk, JOYLINK_PROTO_OBJ_NAME_DEV, JOYLINK_PROTO_SERVICE_OTA, &prop_post);
    prop_post.deviceId = NULL;
    joylink_proto_free_prop_post(&prop_post);

    return JOYLINK_RET_SUCCESS;
}
```
OTA服务内置的4个属性，SDK中的以宏定义给出了示例，开发者可根据实际情况进行更改设置或者维护：

```c
/**
 * @brief  设备ota属性宏定义
 * 
 */
#define D_OTA_PROP_MODEL        ("JDDENG001")	// 设备型号
#define D_OTA_PROP_ACCEPT_DIFF  (false)			// 是否接受差分包
#define D_OTA_PROP_SIG_ALG      (D_OTA_SIG_MD5)	// 设备支持的签名算法

// TODO 如果需要OTA LIST 需要自行按照以下的结构
#ifdef D_OTA_PROP_LIST_REQUIRED
  #define D_OTA_PROP_OTA_LIST      ("[{\"package-name\":\"firmware\", \"version\":\"v1.0.0\", \"diff-base-version\":\"diff-v1.0.0\"}]")
#endif
```
以下章节会对OTA服务内置的4个属性进行一下详细的说明。



#####  5.6.3.1. ota.model

设备型号，必选属性。



#####  5.6.3.2. ota.accept-diff

是否接受差分包，可选属性，bool类型，{true，false}。



#####  5.6.3.3. ota.signature-algorithm

设备OTA支持的签名算法，必选属性。可选值：

```c
 {"1":"SHA256", "2":"SHA256WithRSA","3":"SM3","4":"SM3WithSM2","5":"MD5"}
```



#####  5.6.3.4. ota.ota-list

当前已安装列表，可选属性，类型为数组，成员包括
> [1] package-name：OTA包名,用于区分不用应用，系统固件使用firmware,默认值为firmware；
> [2] version：对应的版本；
> [3] diff-base-version: 对应的差分基包版本；

具体值为以下json数组字符串：
```c
#define D_OTA_PROP_OTA_LIST      ("[{\"package-name\":\"firmware\", \"version\":\"v1.0.0\", \"diff-base-version\":\"diff-v1.0.0\"}]")
```
**附加说明**
> [1] 该属性为可选属性， 如果没有相关的业务需求，可以不用上报；
> [2] 如果有相关要求，开发者需要自行进行列表的维护工作，包括历史安装记录的存储等；并且列表更新后进行一次属性的上报；



#####  5.6.3.2.5. 需要进行OTA属性上报的情形

在如下情况下，开发者需要进行OTA服务属性的上报：
> [1] SDK首次连接物联平台；
> [2] 属性出现变化的时候，比如选择维护ota.ota-list属性的设备，每次OTA升级成功，ota.ota-list更新后，需要将最新的ota.ota-list属性上报至物联管理平台。



####  5.6.4. ota属性获取

如果有业务需求，可参考通用属性来实现，如果没有相关需求，可以不实现。

***



##  6. Example及关键流程说明

### 6.1. 概述
该文档仅仅对提供SDK使用示例及涉及的关键流程逻辑提供相关的说明，API具体的说明及入参，请参阅上一章节。

### 6.2. 章节安排
本说明主要包括以下几个示例流程的说明：
[1] 直连设备；
[2] 网关代理设备；

### 6.3. 直连设备
直连的设备的示例代码路径：example/device，相关主要文件的说明如下：
| 文件名                | 说明                       |
| :-------------------- | :------------------------- |
| test.c                | 主程序入口，主流程示例实现 |
| joylink_device_demo.c | 设备处理逻辑流程示例       |
| joylink_device_ota.c  | ota功能示例                |

#### 6.3.1. 直连设备-主流程
直连设备程序的主要流程如下图所示

![device-main-procdure](images/device-main-procdure.png)



#### 6.3.2. 直连设备-连接成功处理流程

当SDK与物管平台连接成功之后，会调用连接成功的回调函数，通知SDK已经连接成功，在连接成功之后，设备端需要进行如下所示的逻辑处理：

![device-connect](images/device-connect.png)



流程图中，注册连接成功回调函数的动作，建议在SDK进行初始化之前完成，这里仅仅是为了说明，只有注册了该回调函数，在SDK连接成功后，设备才能够获得连接成功的消息。

在连接成功之后，如果设备本身没有时间同步功能，或者当前的网络环境没有进行外部的时钟同步（比如私有化部署）的可能，开发者可以选择使用SDK提供的时间同步接口实现。要求开发者在SDK初始化之前注册相关的回调，在连接成功后，发送请求，物管平台当前的时间会通过回调通知给设备端。

在连接成功后，设备需要将物模行信息、当前的属性值、当前的OTA属性和OTA版本上报给物联平台，完成设备与平台相关信息的同步。

在实际进行处理的过程中，开发者需要进行当前连接状态的维护，并根据实际的应用场景增加网络异常情况下的处理逻辑。

**参考代码 device_task_thread 函数中进入任务循环之前的代码**

```c
    // 示例程序 NTP 授时
    while (true) {
        if (device_demo.is_connected) {
            // 发送时间同步请求
            joylink_sdk_dev_ntp_request(device_demo.sdk);

            // 退出循环
            break;
        }
        log_info("Waitting for SDK Connectted ...");
        jl_platform_msleep(2000);
    }

    // NTP 完成后 进行物模型上报的属性上报
    while (true)
    {
        if ((device_demo.is_connected) && (device_demo.is_ntp_sync)) {
            // 物模型上报
            device_thing_model_post(device_demo.sdk);

            // 属性上报
            device_properties_post(device_demo.sdk);

            // 上报OTA 当前属性
            device_ota_properties_post(device_demo.sdk);

            // TODO OTA 上报当前固件版本
            // char ota_version[32];
            // device_ota_get_version(ota_version);
            // joylink_sdk_dev_ota_version_changed_post(device_demo.sdk, "device", NULL, ota_version);

            // 退出循环
            break;

        }
        if (!device_demo.is_connected) {
            log_info("Waitting for SDK Connectted ...");
        }
        if (!device_demo.is_ntp_sync) {
            log_info("Waitting for SDK Time Synchronized ...");
        }
        jl_platform_msleep(2000);
    }
```





#### 6.3.3.  直连设备-属性设置处理流程

平台对设备的属性更改，SDK会通过属性设置回调函数来与设备进行交互，整个流程如下图所示：

![device-prop-set](images/device-prop-set.png)



设备进行属性设置处理时，需要注意的一点时，需要对属性版本号进行判断，SDK与物管平台约定，只有请求的版本号大于当前的版本号，设备侧才进行属性的更改。在完成正常属性更新后，设备端需要将当前的版本号更新为请求的版本号。

**因此建议开发者在进行代码实现时，需要对版本号进行维护**

**参考代码 回调函数：device_callback_property_set**

```c
int32_t device_callback_property_set(struct joylink_sdk_t *sdk,
                        char *obj_name, char *service_key, JoyProtoPropSet_t *in_set)
{
    // 发送 属性设置响应示例代码
    JoyProtoPropSetRes_t out_res;
    jl_platform_memset(&out_res, 0, sizeof(JoyProtoPropSetRes_t));
    out_res.deviceId = in_set->deviceId;
    out_res.messageId = in_set->messageId;
    out_res.timestamp = jl_platform_time(NULL);
    
    // 检查请求版本号
    if (joylink_sdk_prop_version_check(device_demo.prop_version, in_set->version)) {
        out_res.code = 200;
    }
    else {
        out_res.code = 204;
    }
    
    if (out_res.code == 200) {
        // 检查属性实例名
        if (!jl_platform_strcmp(in_set->properties[0]->key, PROP_KEY)) {
            // 解析期望的值
            joylink_proto_keyvalue_parse_bool(in_set->properties[0]->value, &device_demo.switch_button);
            
            // 更新版本号
            device_demo.prop_version = in_set->version;
            
            // 给出响应
            out_res.prop_num = 1;
            out_res.properties = (JoyProtoKeyValue_t **) jl_platform_malloc(out_res.prop_num * sizeof(JoyProtoKeyValue_t *));
            jl_platform_memset(out_res.properties, 0, out_res.prop_num * sizeof(JoyProtoKeyValue_t *));
            out_res.properties[0] = (JoyProtoKeyValue_t *) jl_platform_malloc(sizeof(JoyProtoKeyValue_t));
            out_res.properties[0]->key = (char *) jl_platform_malloc (32 * sizeof(char));
            jl_platform_strcpy(out_res.properties[0]->key, PROP_KEY);
            out_res.properties[0]->value = joylink_proto_keyvalue_pack_bool(device_demo.switch_button);
            out_res.version = device_demo.prop_version;
        }
        else {
            out_res.code = 400;
        }
    }

    joylink_sdk_dev_prop_set_response(sdk, obj_name, service_key, &out_res);
    
    out_res.deviceId = NULL;
    out_res.messageId = NULL;
    joylink_proto_free_prop_set_res(&out_res);

    return JOYLINK_RET_SUCCESS;
}
```





#### 6.3.4. 直连设备-属性获取处理流程

当平台需要获取设备当前属性值时，SDK会通过属性获取回调函数与设备进行交互，具体流程如下：

![device-prop-get](images/device-prop-get.png)

**参考代码 回调函数：device_callback_property_get**

```c
int32_t device_callback_property_get(struct joylink_sdk_t *sdk,
                        char *obj_name, char *service_key, JoyProtoPropGet_t *in_get)
{
    // 发送 属性设置响应示例代码
    JoyProtoPropGetRes_t out_res;
    jl_platform_memset(&out_res, 0, sizeof(JoyProtoPropGetRes_t));
    out_res.deviceId = in_get->deviceId;
    out_res.messageId = in_get->messageId;
    out_res.timestamp = jl_platform_time(NULL);
    /*
     * out_res.properties[0]->value = joylink_proto_keyvalue_pack_int32(200);
     * 注意内存释放问题
     */
    out_res.prop_num = 1;
    out_res.properties = (JoyProtoKeyValue_t **) jl_platform_malloc(out_res.prop_num * sizeof(JoyProtoKeyValue_t *));
    jl_platform_memset(out_res.properties, 0, out_res.prop_num * sizeof(JoyProtoKeyValue_t *));
    out_res.properties[0] = (JoyProtoKeyValue_t *) jl_platform_malloc(sizeof(JoyProtoKeyValue_t));
    out_res.properties[0]->key = (char *) jl_platform_malloc (32 * sizeof(char));
    jl_platform_strcpy(out_res.properties[0]->key, PROP_KEY);
    out_res.properties[0]->value = joylink_proto_keyvalue_pack_bool(device_demo.switch_button);
    
    out_res.version = device_demo.prop_version;
    out_res.code = 200;

    joylink_sdk_dev_prop_get_response(sdk, obj_name, service_key, &out_res);
    
    out_res.deviceId = NULL;
    out_res.messageId = NULL;
    joylink_proto_free_prop_get_res(&out_res);

    return JOYLINK_RET_SUCCESS;
}
```



#### 6.3.5. 直连设备-方法调用流程

当平台需要调用设备的方法时，SDK会通过方法调用回调函数与设备进行交互，具体流程如下

![device-func-call](images/device-func-call.png)

**参考代码 回调函数 device_callback_function_call**



#### 6.3.6. 直连设备-属性上报

当设备的属性发生变化时，设备需要主动向物管平台进行上报，比如有人通过拨动了一个物理开关。

**在上报的属性中，如果有可读写属性（物管平台可以对该属性进行属性设置），在进行上报时，需要将属性版本号增加1。**

当有实际的业务需求时，设备端也需要进行属性上报。

**参考代码 device_properties_post**

```c
static int32_t device_properties_post(struct joylink_sdk_t *sdk)
{
    // 连接成功 属性上报
    JoyProtoPropPost_t prop_post;
    jl_platform_memset(&prop_post, 0, sizeof(JoyProtoPropPost_t));
    prop_post.deviceId = SDK_CONFIG_DEVICE_ID;
    prop_post.timestamp = jl_platform_time(NULL);
    prop_post.version = device_demo.prop_version;
    prop_post.messageId = joylink_sdk_get_messageId(NULL);

    prop_post.prop_num = 1;
    prop_post.properties = (JoyProtoKeyValue_t **) jl_platform_malloc(prop_post.prop_num * sizeof(JoyProtoKeyValue_t *));
    jl_platform_memset(prop_post.properties, 0, prop_post.prop_num * sizeof(JoyProtoKeyValue_t *));
    prop_post.properties[0] = (JoyProtoKeyValue_t *) jl_platform_malloc(prop_post.prop_num * sizeof(JoyProtoKeyValue_t));
    prop_post.properties[0]->key = (char *) jl_platform_malloc(32 * sizeof(char));
    jl_platform_strcpy(prop_post.properties[0]->key, PROP_KEY);
    prop_post.properties[0]->value = joylink_proto_keyvalue_pack_bool(device_demo.switch_button);

    joylink_sdk_dev_prop_post(sdk, JOYLINK_PROTO_OBJ_NAME_DEV, NULL, &prop_post);
    prop_post.deviceId = NULL;
    joylink_proto_free_prop_post(&prop_post);

    return JOYLINK_RET_SUCCESS;
}
```

#### 6.3.7. 直连设备-事件上报
当设备端定义的事件被触发，需要向物管平台进行上报。
**代码参考**

```c
    jl_time_stamp_t time_stamp;
    char date[32];
    JoyProtoEvtPost_t evt_post;
    jl_platform_memset(&evt_post, 0, sizeof(JoyProtoEvtPost_t));
    evt_post.deviceId = SDK_CONFIG_DEVICE_ID;
    evt_post.evt_num = 1;
    evt_post.events = (JoyProtoEvtPostEvt_t **) jl_platform_malloc (evt_post.evt_num * sizeof(JoyProtoEvtPostEvt_t *));
    evt_post.events[0] = (JoyProtoEvtPostEvt_t *) jl_platform_malloc (sizeof(JoyProtoEvtPostEvt_t));
    jl_platform_memset(evt_post.events[0], 0, sizeof(JoyProtoEvtPostEvt_t));
    evt_post.events[0]->key = (char *) jl_platform_malloc(32 * sizeof(char));
    jl_platform_strcpy(evt_post.events[0]->key, EVENT_KEY);
    evt_post.events[0]->param_num = 1;
    evt_post.events[0]->parameters = (JoyProtoKeyValue_t **) jl_platform_malloc(evt_post.events[0]->param_num * sizeof(JoyProtoKeyValue_t *));
    evt_post.events[0]->parameters[0] = (JoyProtoKeyValue_t *) jl_platform_malloc(sizeof(JoyProtoKeyValue_t));
    jl_platform_memset(evt_post.events[0]->parameters[0], 0, sizeof(JoyProtoKeyValue_t));
    evt_post.events[0]->parameters[0]->key = (char *) jl_platform_malloc(32 * sizeof(char));
    jl_platform_strcpy(evt_post.events[0]->parameters[0]->key, EVENT_ARG_KEY);
    evt_post.events[0]->parameters[0]->value = NULL;
    evt_post.timestamp = jl_platform_time(NULL);
    evt_post.messageId = joylink_sdk_get_messageId(NULL);
            
    // 更新示例时间参数，当前的日期
    jl_platform_time_get_timestamp(&time_stamp);
    jl_platform_sprintf(date, "%d%d", time_stamp.second, time_stamp.ms);
    evt_post.events[0]->parameters[0]->value = joylink_proto_keyvalue_pack_string(date);
            
    joylink_sdk_dev_evt_post(device_demo.sdk, JOYLINK_PROTO_OBJ_NAME_DEV, NULL, &evt_post);
    
	// 释放资源
	evt_post.deviceId = NULL;
    joylink_proto_free_evt_post(&evt_post);
```



### 6.4. 网关设备
网关设备的示例代码路径：example/gateway，相关主要文件说明如下
| 文件名               | 说明                                                         |
| :------------------- | :----------------------------------------------------------- |
| test.c               | 主程序入口，主流程示例实现                                   |
| joylink_device.c     | 网关设备自身处理逻辑流程示例                                 |
| joylink_agent.c      | 代理服务处理逻辑示例，主要子设备的添加、更新、删除、查询方法调用的处理 |
| joylink_sub_device.c | 子设备相关处理流程示例                                       |

在网关代理应用场景中，主要包含网关设备本身，连接代理服务和被代理的子设备，其中网关代理设备本身也是直连设备的一种，因此，网关设备本身的一些功能，比如网关设备本身的属性上报，属性设置、属性获取等功能流程，这一节中不再介绍，开发者可以参考直连设备的。



#### 6.4.1 网关设备-主流程

网关主流程与直连设备一致，区别在设备回调函数注册部分，需要新增connection_agent 服务方法调用的回调函数注册
```c
    // 设置连接代理 方法调用回调函数
    joylink_sdk_set_agent_cb_func_call(joy_sdk, agent_callback_function_call);
```
和子设备处理逻辑相关的回调函数
```c
    // 设置子设备 属性设置回调函数
    joylink_sdk_set_sub_dev_cb_prop_set(joy_sdk, sub_device_callback_property_set);

    // 设置子设备 属性获取回调函数
    joylink_sdk_set_sub_dev_cb_prop_get(joy_sdk, sub_device_callback_property_get);

    // 设置子设备 方法调用回调函数
    joylink_sdk_set_sub_dev_cb_func_call(joy_sdk, sub_device_callback_function_call);
```



#### 6.4.2. 网关设备-连接成功处理逻辑

网关设备连接成功之后，处理的逻辑流程如下图所示：

![gateway-connect-callback](images/gateway-connect-callback.png)

与直连设备的相比，在连接成功后，网关设备需要代理子设备进行上下线状态的上报以及子设备当前属性的上报。

**参考代码**

```c
int32_t device_callback_connect(struct joylink_sdk_t *sdk)
{
    // 物模型上报
    device_thing_model_post(sdk);

    // 上报自身属性
    device_properties_post(sdk);

    // 设备OTA属性上报
    device_ota_properties_post(sdk);

    // 设备OTA版本号上报
    char ota_version[32];
    device_ota_get_version(ota_version);
    joylink_sdk_dev_ota_version_changed_post(sdk, "device", NULL, ota_version);

    // 代理子设备进行上线状态上报
    joylink_sdk_sub_dev_online_post(sdk, "device", sub_device_get_info(NULL));

    // 用户可根据自身情况进行逻辑处理
    device_demo.is_connected = 1;

    return JOYLINK_RET_SUCCESS;
}
```



#### 6.4.3. 网关设备-代理服务方法调用流程

网关代理服务，及connection_agent 服务，内置了4个方法调用（添加子设备、更新子设备、删除子设备和查询子设备信息），用来实现对子设备的管理。整体的处理逻辑流程如下所示：

![gateway-agent-func-call](images/gateway-agent-func-call.png)

**参考代码**

```c
int32_t agent_callback_function_call(struct joylink_sdk_t *sdk, JoyProtoFuncCall_t *in_call)
{
    JoyProtoFuncCallRes_t call_res;
    JoyProtoFuncCallRes_t *out_res = &call_res;
    jl_platform_memset(out_res, 0, sizeof(JoyProtoFuncCallRes_t));

    // 用于解析子设备信息
    JoyProtoSubDeviceInfo_t subDev;
    jl_platform_memset(&subDev, 0, sizeof(JoyProtoSubDeviceInfo_t));

    out_res->func_num = in_call->func_num;
    out_res->code = 200;
    out_res->functions = (JoyProtoFuncCallResFunc_t **) jl_platform_malloc(
            out_res->func_num * sizeof(JoyProtoFuncCallResFunc_t *));
    jl_platform_memset(out_res->functions, 0, out_res->func_num * sizeof(JoyProtoFuncCallResFunc_t *));

    // 添加子设备
    if (!jl_platform_strcmp(in_call->functions[0]->key, JOYLINK_PROTO_AGENT_FUNC_ADD)) {
        // parse subdevice info
        joylink_proto_free_sub_dev_info(&subDev);
        joylink_proto_keyvalue_parse_subdevinfo(in_call->functions[0]->in[0]->value, &subDev);
        out_res->functions[0] = agent_process_func_call_add_devices(&subDev);
    }
    // 删除子设备
    else if (!jl_platform_strcmp(in_call->functions[0]->key, JOYLINK_PROTO_AGENT_FUNC_DEL)) {
        // parse subdevice info
        joylink_proto_free_sub_dev_info(&subDev);
        joylink_proto_keyvalue_parse_subdevinfo(in_call->functions[0]->in[0]->value, &subDev);
        out_res->functions[0] = agent_process_func_call_del_devices(&subDev);                
    }
    // 更新子设备
    else if (!jl_platform_strcmp(in_call->functions[0]->key, JOYLINK_PROTO_AGENT_FUNC_UPDATE)) {
        // parse subdevice info
        joylink_proto_free_sub_dev_info(&subDev);
        joylink_proto_keyvalue_parse_subdevinfo(in_call->functions[0]->in[0]->value, &subDev);
        out_res->functions[0] = agent_process_func_call_update_devices(&subDev);                
    }
    // 获取子设备
    else if (!jl_platform_strcmp(in_call->functions[0]->key, JOYLINK_PROTO_AGENT_FUNC_GET)) {
        out_res->functions[0] = agent_process_func_call_get_devices();                
    }
    else {
        log_error("Unsupport Connection Agent Function Call");
        out_res->code = 601;
    }

    // 释放资源 子设备信息
    joylink_proto_free_sub_dev_info(&subDev);

    // 发送响应
    out_res->deviceId = in_call->deviceId;
    out_res->messageId = in_call->messageId;
    out_res->timestamp = jl_platform_time(NULL);
    joylink_sdk_agent_func_call_response(sdk, out_res);

    // 释放资源 响应报文
    out_res->deviceId = NULL;
    out_res->messageId = NULL;
    joylink_proto_free_func_call_res(out_res);

    return JOYLINK_RET_SUCCESS;
}
```



#### 6.4.4. 网关设备-子设备-属性设置

网关代理子设备的属性设置处理流程如下：

![gateway-sub-prop-set](images/gateway-sub-prop-set.png)

**参考代码**

```c
int32_t sub_device_callback_property_set(struct joylink_sdk_t *sdk, JoyProtoPropSet_t *in_set)
{
    // 发送 属性设置响应示例代码
    JoyProtoPropSetRes_t out_res;
    jl_platform_memset(&out_res, 0, sizeof(JoyProtoPropSetRes_t));
    out_res.deviceId = in_set->deviceId;
    out_res.messageId = in_set->messageId;
    out_res.timestamp = jl_platform_time(NULL);
    
    // 检查请求版本号
    if (joylink_sdk_prop_version_check(sub_device.prop_version, in_set->version)) {
        out_res.code = 200;
    }
    else {
        out_res.code = 204;
    }
    
    if (out_res.code == 200) {
        // 检查属性实例名
        if (!jl_platform_strcmp(in_set->properties[0]->key, PROP_KEY)) {
            // 解析期望的值
            joylink_proto_keyvalue_parse_bool(in_set->properties[0]->value, &sub_device.light_power);
            
            // 更新版本号
            sub_device.prop_version = in_set->version;
            
            // 给出响应
            out_res.prop_num = 1;
            out_res.properties = (JoyProtoKeyValue_t **) jl_platform_malloc(out_res.prop_num * sizeof(JoyProtoKeyValue_t *));
            jl_platform_memset(out_res.properties, 0, out_res.prop_num * sizeof(JoyProtoKeyValue_t *));
            out_res.properties[0] = (JoyProtoKeyValue_t *) jl_platform_malloc(sizeof(JoyProtoKeyValue_t));
            out_res.properties[0]->key = (char *) jl_platform_malloc (32 * sizeof(char));
            jl_platform_strcpy(out_res.properties[0]->key, PROP_KEY);
            out_res.properties[0]->value = joylink_proto_keyvalue_pack_bool(sub_device.light_power);
            out_res.version = sub_device.prop_version;
        }
        else {
            out_res.code = 400;
        }
    }

    joylink_sdk_sub_dev_prop_set_response(sdk, &out_res);
    
    out_res.deviceId = NULL;
    out_res.messageId = NULL;
    joylink_proto_free_prop_set_res(&out_res);

    return JOYLINK_RET_SUCCESS;
}
```



#### 6.4.5. 网关设备-子设备-属性获取
网关代理子设备的属性获取处理流程如下：

![gateway-sub-prop-get](images/gateway-sub-prop-get.png)

**参考代码**

```c
int32_t sub_device_callback_property_get(struct joylink_sdk_t *sdk, JoyProtoPropGet_t *in_get)
{
    // 发送 属性设置响应示例代码
    JoyProtoPropGetRes_t out_res;
    jl_platform_memset(&out_res, 0, sizeof(JoyProtoPropGetRes_t));
    out_res.deviceId = in_get->deviceId;
    out_res.messageId = in_get->messageId;
    out_res.timestamp = jl_platform_time(NULL);
    /*
     * out_res.properties[0]->value = joylink_proto_keyvalue_pack_int32(200);
     * 注意内存释放问题
     */
    out_res.prop_num = 1;
    out_res.properties = (JoyProtoKeyValue_t **) jl_platform_malloc(out_res.prop_num * sizeof(JoyProtoKeyValue_t *));
    jl_platform_memset(out_res.properties, 0, out_res.prop_num * sizeof(JoyProtoKeyValue_t *));
    out_res.properties[0] = (JoyProtoKeyValue_t *) jl_platform_malloc(sizeof(JoyProtoKeyValue_t));
    out_res.properties[0]->key = (char *) jl_platform_malloc (32 * sizeof(char));
    jl_platform_strcpy(out_res.properties[0]->key, PROP_KEY);
    out_res.properties[0]->value = joylink_proto_keyvalue_pack_bool(sub_device.light_power);
    
    out_res.version = sub_device.prop_version;
    out_res.code = 200;

    joylink_sdk_sub_dev_prop_get_response(sdk, &out_res);
    
    out_res.deviceId = NULL;
    out_res.messageId = NULL;
    joylink_proto_free_prop_get_res(&out_res);
    return JOYLINK_RET_SUCCESS;
}
```



#### 6.4.6. 网关设备-子设备-方法调用

网关代理子设备的方法调用处理流程如下：

![gateway-sub-func-call](images/gateway-sub-func-call.png)



#### 6.4.7. 网关设备-子设备-属性上报

当子设备需要进行属性上报时，可通过网关设备代理进行上报。

**参考代码**

```c
int32_t sub_device_properties_post(struct joylink_sdk_t *sdk, char *subdeviceId)
{
    JoyProtoPropPost_t prop_post;
    jl_platform_memset(&prop_post, 0, sizeof(JoyProtoPropPost_t));
    prop_post.deviceId = sub_device.info.deviceId;
    prop_post.timestamp = jl_platform_time(NULL);
    prop_post.version = sub_device.prop_version;
    prop_post.messageId = joylink_sdk_get_messageId(NULL);

    prop_post.prop_num = 1;
    prop_post.properties = (JoyProtoKeyValue_t **) jl_platform_malloc(prop_post.prop_num * sizeof(JoyProtoKeyValue_t *));
    jl_platform_memset(prop_post.properties, 0, prop_post.prop_num * sizeof(JoyProtoKeyValue_t *));
    prop_post.properties[0] = (JoyProtoKeyValue_t *) jl_platform_malloc(prop_post.prop_num * sizeof(JoyProtoKeyValue_t));
    prop_post.properties[0]->key = (char *) jl_platform_malloc(32 * sizeof(char));
    jl_platform_strcpy(prop_post.properties[0]->key, PROP_KEY);
    prop_post.properties[0]->value = joylink_proto_keyvalue_pack_bool(sub_device.light_power);

    joylink_sdk_sub_dev_prop_post(sdk, &prop_post);
    prop_post.deviceId = NULL;
    joylink_proto_free_prop_post(&prop_post);

    return JOYLINK_RET_SUCCESS;
}
```



#### 6.4.8. 网关设备-子设备-事件上报

当子设备需要进行事件上报时，可通过网关设备代理进行上报。

**参考代码**

```c
int32_t sub_device_event_post(struct joylink_sdk_t *sdk, char *subdeviceId)
{
    jl_time_stamp_t time_stamp;
    char date[32];

    JoyProtoEvtPost_t evt_post;
    jl_platform_memset(&evt_post, 0, sizeof(JoyProtoEvtPost_t));
    evt_post.deviceId = sub_device.info.deviceId;
    evt_post.timestamp = jl_platform_time(NULL);
    evt_post.messageId = joylink_sdk_get_messageId(NULL);
    evt_post.evt_num = 1;
    evt_post.events = (JoyProtoEvtPostEvt_t **) jl_platform_malloc (evt_post.evt_num * sizeof(JoyProtoEvtPostEvt_t *));
    evt_post.events[0] = (JoyProtoEvtPostEvt_t *) jl_platform_malloc (sizeof(JoyProtoEvtPostEvt_t));
    jl_platform_memset(evt_post.events[0], 0, sizeof(JoyProtoEvtPostEvt_t));
    evt_post.events[0]->key = (char *) jl_platform_malloc(32 * sizeof(char));
    jl_platform_strcpy(evt_post.events[0]->key, EVENT_KEY);
    evt_post.events[0]->param_num = 1;
    evt_post.events[0]->parameters = (JoyProtoKeyValue_t **) jl_platform_malloc(evt_post.events[0]->param_num * sizeof(JoyProtoKeyValue_t *));
    evt_post.events[0]->parameters[0] = (JoyProtoKeyValue_t *) jl_platform_malloc(sizeof(JoyProtoKeyValue_t));
    jl_platform_memset(evt_post.events[0]->parameters[0], 0, sizeof(JoyProtoKeyValue_t));
    evt_post.events[0]->parameters[0]->key = (char *) jl_platform_malloc(32 * sizeof(char));
    jl_platform_strcpy(evt_post.events[0]->parameters[0]->key, EVENT_ARG_KEY);

    jl_platform_time_get_timestamp(&time_stamp);
    jl_platform_sprintf(date, "%d%d", time_stamp.second, time_stamp.ms);
    evt_post.events[0]->parameters[0]->value = joylink_proto_keyvalue_pack_string(date);

    // 上报事件
    joylink_sdk_sub_dev_evt_post(sdk, &evt_post);

    // 释放资源
    evt_post.deviceId = NULL;
    joylink_proto_free_evt_post(&evt_post);
}
```

-------------文档结束-------------

* * *
