/*Copyright (c) 2015-2050, JD All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License. */

/* --------------------------------------------------
 * @brief: 头文件 设备ota示例
 *
 * @version: 1.0
 *
 * --------------------------------------------------
 */

#ifndef _JOYLINK_DEVICE_OTA_H_
#define _JOYLINK_DEVICE_OTA_H_

#include <stdio.h>
#include "joylink.h"
#include "joylink_protocol.h"
#include "joylink_sdk.h"

#include "joylink_stdint.h"


#ifdef __cplusplus
extern "C"{
#endif /* __cplusplus */

/**
 * @brief  设备ota方法调用参数实例名宏定义
 * 
 */
#define D_OTA_FUNC_ARG_TASK_ID              JOYLINK_PROTO_OTA_TASK_ID       // 任务标识
#define D_OTA_FUNC_ARG_PKG_NAME             JOYLINK_PROTO_OTA_PKG_NAME      // OTA 包名
#define D_OTA_FUNC_ARG_NAME                 ("name")                        // 升级包版本名称
#define D_OTA_FUNC_ARG_VERSION              JOYLINK_PROTO_OTA_VERSION       // 升级包版本号
#define D_OTA_FUNC_ARG_OBJECT_ID            JOYLINK_PROTO_OTA_OBJECT_ID     // 在对子设备升级时，此为子设备id
#define D_OTA_FUNC_ARG_OBJECT_TYPE          ("object-type")                 // 升级对象类型 {"app", "firmware", "framework", "os"}
#define D_OTA_FUNC_ARG_URL                  ("url")                         // 全量包获取网址
#define D_OTA_FUNC_ARG_INSTALL_MODE         ("install-mode")                // 升级模式
#define D_OTA_FUNC_ARG_SIZE                 ("size")                        // 升级全量包大小，以字节为单位
#define D_OTA_FUNC_ARG_CUSTOMIZED           ("customized")                  // 用户自定义的辅助信息
#define D_OTA_FUNC_ARG_TOKEN                ("token")                       // 访问升级包的令牌
#define D_OTA_FUNC_ARG_SIGNATURE            ("signature")                   // 升级包签名
#define D_OTA_FUNC_ARG_DIFF_BASE_VERSION    ("diff-base-version")           // 差分基包版本，无差分包时为空
#define D_OTA_FUNC_ARG_DIFF_BASE_SIGNATURE  ("diff-base-signature")         // 差分基包签名，无差分包时为空
#define D_OTA_FUNC_ARG_DIFF_URL             ("diff-url")                    // 差分包url，无差分包时为空
#define D_OTA_FUNC_ARG_DIFF_SIZE            ("diff-size")                   // 差分包的大小，以字节为单位
#define D_OTA_FUNC_ARG_DIFF_TOKEN           ("diff-token")                  // 访问差分包的令牌
#define D_OTA_FUNC_ARG_DIFF_SIGNATURE       ("diff-signature")              // 差分包签名，无差分包时为空
#define D_OTA_FUNC_ARG_SIG_ALG              ("signature-algorithm")         // 升级包签名算法
#define D_OTA_FUNC_ARG_TIME                 ("time")                        // 升级执行时间

/**
 * @brief  设备ota升级对象类型
 * 
 * @note D_OTA_FUNC_ARG_OBJECT_TYPE 对应的枚举分类
 * 
 */
#define D_OTA_OBJECT_TYPE_APP               ("app")
#define D_OTA_OBJECT_TYPE_FIRMWARE          ("firmware")
#define D_OTA_OBJECT_TYPE_FRAMEWORK         ("framework")
#define D_OTA_OBJECT_TYPE_OS                ("os")

/**
 * @brief  设备ota升级模式
 * 
 * @note D_OTA_FUNC_ARG_INSTALL_MODE 对应的枚举分类
 * 
 */
#define D_OTA_INSTALL_MODE_IMM       ("0")      // 立即安装
#define D_OTA_INSTALL_MODE_C_D_I     ("1")      // 用户确认后下载安装
#define D_OTA_INSTALL_MODE_C_I       ("2")      // 用户确认后安装
#define D_OTA_INSTALL_MODE_TIMING    ("3")      // 定时安装
#define D_OTA_INSTALL_MODE_IDLE      ("4")      // 闲时安装

/**
 * @brief  设备ota签名算法枚举类
 * 
 */
#define D_OTA_SIG_MD5                       ("0")
#define D_OTA_SIG_ALG_SHA256                ("1")
#define D_OTA_SIG_ALG_SHA256WITHRSA         ("2")
#define D_OTA_SIG_SM3                       ("3")
#define D_OTA_SIG_SM3WITHSM2                ("4")

/**
 * @brief  是否需要 ota-list 支持
 * 
 */
#define D_OTA_PROP_LIST_REQUIRED

/**
 * @brief  设备ota属性宏定义
 * 
 */
#define D_OTA_PROP_MODEL        ("JDDENG001")
#define D_OTA_PROP_ACCEPT_DIFF  (false)
#define D_OTA_PROP_SIG_ALG      (D_OTA_SIG_MD5)

// TODO 如果需要OTA LIST 需要自行按照以下的结构
#ifdef D_OTA_PROP_LIST_REQUIRED
  #define D_OTA_PROP_OTA_LIST      ("[{\"package-name\":\"firmware\",\"version\":\"v1.0.0\", \"diff-base-version\":\"diff-v1.0.0\"}]")
#endif


/**
 * @brief  ota 获取当前固件版本号
 * 
 * @param[in] version: 输出版本号指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note None.
 *
 */
int32_t device_ota_get_version(char *version);

/**
 * @brief  ota 获取当前OTA属性
 * 
 * @return 
 * @note None.
 *
 */
JoyProtoKeyValue_t **device_ota_get_properties(int32_t *number);

/**
 * @brief  处理ota方法调用
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @param[in] obj_name: 对象空间，可选值{"device", "edge", "group"}
 * @param[in] in_call: 方法调用消息结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note None.
 *
 */
int32_t device_ota_process_func_call(struct joylink_sdk_t *sdk, char *obj_name, JoyProtoFuncCall_t *in_call);

/**
 * @brief  处理ota方法调用 ota
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @param[in] obj_name: 对象空间，可选值{"device", "edge", "group"}
 * @param[in] in_call: 方法调用消息结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note None.
 *
 */
int32_t device_ota_process_func_ota(struct joylink_sdk_t *sdk, char *obj_name, JoyProtoFuncCall_t *in_call);

/**
 * @brief  处理ota方法调用 查询状态
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @param[in] obj_name: 对象空间，可选值{"device", "edge", "group"}
 * @param[in] in_call: 方法调用消息结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note None.
 *
 */
int32_t device_ota_process_func_query_state(struct joylink_sdk_t *sdk, char *obj_name, JoyProtoFuncCall_t *in_call);

/**
 * @brief  处理ota方法调用 取消任务
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @param[in] obj_name: 对象空间，可选值{"device", "edge", "group"}
 * @param[in] in_call: 方法调用消息结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note None.
 *
 */
int32_t device_ota_process_func_cancel_task(struct joylink_sdk_t *sdk, char *obj_name, JoyProtoFuncCall_t *in_call);

/**
 * @brief  设备ota模拟任务线程
 * 
 * @note None.
 *
 */
void device_ota_task_thread(void *in_param);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _JOYLINK_DEVICE_OTA_H_ */
