/*Copyright (c) 2015-2050, JD All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License. */

/* --------------------------------------------------
 * @brief: 源文件 一个设备示例
 *
 * @version: 1.0
 *
 * --------------------------------------------------
 */

#include <stdio.h>
#include "joylink_agent.h"
#include "joylink_string.h"
#include "joylink_memory.h"
#include "joylink_log.h"
#include "joylink_time.h"
#include "joylink_thread.h"

#include "joylink_protocol.h"
#include "joylink_sdk.h"

#include "cJSON.h"

// 定义当前的子设备信息
// 这里仅仅是示例程序，为开发者提供API使用流程的示例
// 开发者需要根据应用场景和硬件资源，自行去维护该信息
static JoyProtoSubDeviceInfo_t subDevCache = {0};

/**
 * @brief  连接代理初始化
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note
 */
int32_t agent_initialise(struct joylink_sdk_t *sdk)
{
    return JOYLINK_RET_SUCCESS;
}

/**
 * @brief   connection agent服务 add-devices 方法调用 处理函数
 *
 * @param[in] in_info: 子设备列表信息
 * @return 
 *    方法调用响应结果体指针
 * @see None.
 * @note None.
 */
static JoyProtoFuncCallResFunc_t *agent_process_func_call_add_devices(JoyProtoSubDeviceInfo_t *in_info)
{
    log_info("Enter agent_process_func_call_add_devices");
    JoyProtoFuncCallResFunc_t *out = NULL;

    // 处理示例流程
    JoyProtoSubDevice_t **info_tmp;
    int ii, cur_number;
    log_info("in_info subdevice number:%d", in_info->dev_num);
    if (in_info->dev_num > 0) {
        cur_number = subDevCache.dev_num;

        info_tmp = (JoyProtoSubDevice_t **) jl_platform_malloc(
            (cur_number + in_info->dev_num)* sizeof(JoyProtoSubDevice_t *));
        if (cur_number > 0) {
            for (ii = 0; ii < cur_number; ii++) {
                info_tmp[ii] = subDevCache.subDev[ii];
            }
            jl_platform_free(subDevCache.subDev);
            subDevCache.subDev = NULL;
        }

        for (ii = 0; ii < in_info->dev_num; ii++) {
            info_tmp[ii + cur_number] = (JoyProtoSubDevice_t *) jl_platform_malloc(sizeof(JoyProtoSubDevice_t));
            jl_platform_memset(info_tmp[ii + cur_number], 0, sizeof(JoyProtoSubDevice_t));
            joylink_proto_copy_sub_dev(info_tmp[ii + cur_number], in_info->subDev[ii]);
        }
        subDevCache.dev_num = cur_number + in_info->dev_num;
        subDevCache.subDev = info_tmp;
        log_info("in cache current subdevice number is:%d", subDevCache.dev_num);
        for (ii = 0; ii < subDevCache.dev_num; ii++) {
            log_info("in cache subdevice[%d]->deviceId:%s", ii, subDevCache.subDev[ii]->deviceId);
        }

        // TODO 开发着可以将添加后的信息 保存到数据库或者文件

        // response out
        out = joylink_proto_agent_create_opsub_func_Res(JOYLINK_PROTO_AGENT_FUNC_ADD, NULL,200, "ok");
    } else {
        out = joylink_proto_agent_create_opsub_func_Res(JOYLINK_PROTO_AGENT_FUNC_ADD, NULL,204, "the device list is empty");
    }
    log_info("Leave agent_process_func_call_add_devices");
    return out;
}

/**
 * @brief   connection agent服务 delete-devices 方法调用 处理函数
 *
 * @param[in] in_info: 子设备列表信息
 * @return 
 *    方法调用响应结果体指针
 * @see None.
 * @note None.
 */
static JoyProtoFuncCallResFunc_t *agent_process_func_call_del_devices(JoyProtoSubDeviceInfo_t *in_info)
{
    log_info("Enter agent_process_func_call_del_devices");
    JoyProtoFuncCallResFunc_t *out = NULL;

    // process
    JoyProtoSubDevice_t **info_tmp;
    int ii, jj, cur_number, index, foundFlag;
    log_info("in_info subdevice number:%d", in_info->dev_num);
    cur_number = subDevCache.dev_num;
    if (in_info->dev_num < cur_number) {
        info_tmp = (JoyProtoSubDevice_t **) jl_platform_malloc(
            (cur_number - in_info->dev_num)* sizeof(JoyProtoSubDevice_t *));
        index = 0;    
        for (ii = 0; ii < cur_number; ii++) {
            foundFlag = 0;
            for (jj = 0; jj < in_info->dev_num; jj++) {
                if (!jl_platform_strcmp(subDevCache.subDev[ii]->deviceId, in_info->subDev[jj]->deviceId)) {
                    foundFlag = 1;
                    break;
                }
                if (foundFlag) {
                    // 如果找到 删除
                    joylink_proto_free_sub_dev(subDevCache.subDev[ii]);
                    jl_platform_free(subDevCache.subDev[ii]);
                    subDevCache.subDev[ii] = NULL;
                } else {
                    // 没有找到 保留
                    info_tmp[index] = subDevCache.subDev[ii];
                    index++;
                }
            }
        }
        jl_platform_free(subDevCache.subDev);
        subDevCache.subDev = NULL;

        subDevCache.dev_num = cur_number - in_info->dev_num;
        subDevCache.subDev = info_tmp;
        log_info("in cache current subdevice number is:%d", subDevCache.dev_num);
        for (ii = 0; ii < subDevCache.dev_num; ii++) {
            log_info("in cache subdevice[%d]->deviceId:%s", ii, subDevCache.subDev[ii]->deviceId);
        }

        // TODO 开发着可以将添加后的信息 保存到数据库或者文件

        // response out
        out = joylink_proto_agent_create_opsub_func_Res(JOYLINK_PROTO_AGENT_FUNC_DEL, NULL,200, "ok");
    }
    else if (in_info->dev_num == cur_number) {
        // 全部删除
        for (ii = 0; ii < cur_number; ii++) {
            joylink_proto_free_sub_dev(subDevCache.subDev[ii]);
            jl_platform_free(subDevCache.subDev[ii]);
            subDevCache.subDev[ii] = NULL;
        }
        jl_platform_free(subDevCache.subDev);
        subDevCache.subDev = NULL;
        subDevCache.dev_num = 0;
        log_info("in cache there is no sub device info");

        // TODO 开发着可以将添加后的信息 保存到数据库或者文件

        out = joylink_proto_agent_create_opsub_func_Res(JOYLINK_PROTO_AGENT_FUNC_DEL, NULL,200, "ok");        
    } 
    else {
        out = joylink_proto_agent_create_opsub_func_Res(JOYLINK_PROTO_AGENT_FUNC_DEL, NULL, 204, "do not hava enough device");
    }
    log_info("Leave agent_process_func_call_del_devices");
    return out;
}

/**
 * @brief   connection agent服务 update-devices 方法调用 处理函数
 *
 * @param[in] in_info: 子设备列表信息
 * @return 
 *    方法调用响应结果体指针
 * @see None.
 * @note None.
 */
static JoyProtoFuncCallResFunc_t *agent_process_func_call_update_devices(JoyProtoSubDeviceInfo_t *in_info)
{
    log_info("Enter agent_process_func_call_update_devices");
    JoyProtoFuncCallResFunc_t *out = NULL;

    // 处理流程示例
    JoyProtoSubDevice_t **info_tmp;
    int ii, cur_number;
    log_info("in_info subdevice number:%d", in_info->dev_num);
    if (in_info->dev_num > 0) {
        cur_number = subDevCache.dev_num;
        
        
        info_tmp = (JoyProtoSubDevice_t **) jl_platform_malloc(
            (in_info->dev_num)* sizeof(JoyProtoSubDevice_t *));
        if (cur_number > 0) {
            for (ii = 0; ii < cur_number; ii++) {
                // remove
                joylink_proto_free_sub_dev(subDevCache.subDev[ii]);
                jl_platform_free(subDevCache.subDev[ii]);
                subDevCache.subDev[ii] = NULL;
            }
            jl_platform_free(subDevCache.subDev);
            subDevCache.subDev = NULL;
        }

        for (ii = 0; ii < in_info->dev_num; ii++) {
            info_tmp[ii] = (JoyProtoSubDevice_t *) jl_platform_malloc(sizeof(JoyProtoSubDevice_t));
            jl_platform_memset(info_tmp[ii], 0, sizeof(JoyProtoSubDevice_t));
            joylink_proto_copy_sub_dev(info_tmp[ii], in_info->subDev[ii]);
        }
        subDevCache.dev_num = in_info->dev_num;
        subDevCache.subDev = info_tmp;
        log_info("in cache current subdevice number is:%d", subDevCache.dev_num);
        for (ii = 0; ii < subDevCache.dev_num; ii++) {
            log_info("in cache subdevice[%d]->deviceId:%s", ii, subDevCache.subDev[ii]->deviceId);
        }

        // TODO 开发着可以将添加后的信息 保存到数据库或者文件

        // response out
        out = joylink_proto_agent_create_opsub_func_Res(JOYLINK_PROTO_AGENT_FUNC_UPDATE, NULL,200, "ok");
    } else {
        out = joylink_proto_agent_create_opsub_func_Res(JOYLINK_PROTO_AGENT_FUNC_UPDATE, NULL,204, "the device list is empty");
    }
    log_info("Leave agent_process_func_call_update_devices");
    return out;
}

/**
 * @brief   connection agent服务 get-devices 方法调用 处理函数
 *
 * @param[in] None.
 * @return 
 *    方法调用响应结果体指针
 * @see None.
 * @note None.
 */
static JoyProtoFuncCallResFunc_t *agent_process_func_call_get_devices()
{
    log_info("Enter agent_process_func_call_get_devices");
    JoyProtoFuncCallResFunc_t *out = NULL;
    out = joylink_proto_agent_create_opsub_func_Res(
                            JOYLINK_PROTO_AGENT_FUNC_GET, 
                            &subDevCache,
                            200, 
                            "ok");

    log_info("Leave agent_process_func_call_get_devices");
    return out;
}

/**
 * @brief  方法调用 回调函数
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @param[in] in_call: 方法调用消息结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note
 *    sdk 收到方法调用消息时 会调用该函数
 */
int32_t agent_callback_function_call(struct joylink_sdk_t *sdk, JoyProtoFuncCall_t *in_call)
{
    JoyProtoFuncCallRes_t call_res;
    JoyProtoFuncCallRes_t *out_res = &call_res;
    jl_platform_memset(out_res, 0, sizeof(JoyProtoFuncCallRes_t));

    // 用于解析子设备信息
    JoyProtoSubDeviceInfo_t subDev;
    jl_platform_memset(&subDev, 0, sizeof(JoyProtoSubDeviceInfo_t));

    out_res->func_num = in_call->func_num;
    out_res->code = 200;
    out_res->functions = (JoyProtoFuncCallResFunc_t **) jl_platform_malloc(
            out_res->func_num * sizeof(JoyProtoFuncCallResFunc_t *));
    jl_platform_memset(out_res->functions, 0, out_res->func_num * sizeof(JoyProtoFuncCallResFunc_t *));

    // 添加子设备
    if (!jl_platform_strcmp(in_call->functions[0]->key, JOYLINK_PROTO_AGENT_FUNC_ADD)) {
        // parse subdevice info
        joylink_proto_free_sub_dev_info(&subDev);
        joylink_proto_keyvalue_parse_subdevinfo(in_call->functions[0]->in[0]->value, &subDev);
        out_res->functions[0] = agent_process_func_call_add_devices(&subDev);
    }
    // 删除子设备
    else if (!jl_platform_strcmp(in_call->functions[0]->key, JOYLINK_PROTO_AGENT_FUNC_DEL)) {
        // parse subdevice info
        joylink_proto_free_sub_dev_info(&subDev);
        joylink_proto_keyvalue_parse_subdevinfo(in_call->functions[0]->in[0]->value, &subDev);
        out_res->functions[0] = agent_process_func_call_del_devices(&subDev);                
    }
    // 更新子设备
    else if (!jl_platform_strcmp(in_call->functions[0]->key, JOYLINK_PROTO_AGENT_FUNC_UPDATE)) {
        // parse subdevice info
        joylink_proto_free_sub_dev_info(&subDev);
        joylink_proto_keyvalue_parse_subdevinfo(in_call->functions[0]->in[0]->value, &subDev);
        out_res->functions[0] = agent_process_func_call_update_devices(&subDev);                
    }
    // 获取子设备
    else if (!jl_platform_strcmp(in_call->functions[0]->key, JOYLINK_PROTO_AGENT_FUNC_GET)) {
        out_res->functions[0] = agent_process_func_call_get_devices();                
    }
    else {
        log_error("Unsupport Connection Agent Function Call");
        out_res->code = 601;
    }

    // 释放资源 子设备信息
    joylink_proto_free_sub_dev_info(&subDev);

    // 发送响应
    out_res->deviceId = in_call->deviceId;
    out_res->messageId = in_call->messageId;
    out_res->timestamp = jl_platform_time(NULL);
    joylink_sdk_agent_func_call_response(sdk, out_res);

    // 释放资源 响应报文
    out_res->deviceId = NULL;
    out_res->messageId = NULL;
    joylink_proto_free_func_call_res(out_res);

    return JOYLINK_RET_SUCCESS;
}

/**
 * @brief  连接代理 非直连设备绑定事件
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note
 *    none
 */
int32_t agent_event_topology_add_devices(struct joylink_sdk_t *sdk)
{
    JoyProtoSubDeviceInfo_t sub_dev_info = {0};
    sub_dev_info.dev_num = 1;
    sub_dev_info.subDev = (JoyProtoSubDevice_t **) jl_platform_malloc(sub_dev_info.dev_num * sizeof(JoyProtoSubDevice_t *));
    jl_platform_memset(sub_dev_info.subDev, 0, sub_dev_info.dev_num * sizeof(JoyProtoSubDevice_t *));

    JoyProtoSubDevice_t sub_device = {0};
    sub_device.name = "sub_light";
    sub_device.deviceId = "0001";
    sub_dev_info.subDev[0] = &sub_device;

    joylink_sdk_agent_event_add_devices(sdk, &sub_dev_info);

    jl_platform_free(sub_dev_info.subDev);

    return JOYLINK_RET_SUCCESS;
}

/**
 * @brief  连接代理 非直连设备解绑事件
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note
 *    none
 */
int32_t agent_event_topology_delete_devices(struct joylink_sdk_t *sdk)
{
    JoyProtoSubDeviceInfo_t sub_dev_info = {0};
    sub_dev_info.dev_num = 1;
    sub_dev_info.subDev = (JoyProtoSubDevice_t **) jl_platform_malloc(sub_dev_info.dev_num * sizeof(JoyProtoSubDevice_t *));
    jl_platform_memset(sub_dev_info.subDev, 0, sub_dev_info.dev_num * sizeof(JoyProtoSubDevice_t *));

    JoyProtoSubDevice_t sub_device = {0};
    sub_device.name = "sub_light";
    sub_device.deviceId = "0000001";
    sub_dev_info.subDev[0] = &sub_device;

    joylink_sdk_agent_event_delete_devices(sdk, &sub_dev_info);

    jl_platform_free(sub_dev_info.subDev);

    return JOYLINK_RET_SUCCESS;
}

// end of file
