/*Copyright (c) 2015-2050, JD All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License. */

/* --------------------------------------------------
 * @brief: 头文件 一个设备示例
 *
 * @version: 1.0
 *
 * --------------------------------------------------
 */

#ifndef _JOYLINK_SUB_DEVICE_H_
#define _JOYLINK_SUB_DEVICE_H_

#include <stdio.h>
#include "joylink.h"
#include "joylink_protocol.h"
#include "joylink_sdk.h"

#include "joylink_stdint.h"
#include "joylink_time.h"


#ifdef __cplusplus
extern "C"{
#endif /* __cplusplus */

/**
 * @brief  子设备初始化
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note
 */
int32_t sub_device_initialise(struct joylink_sdk_t *sdk);

/**
 * @brief  属性设置 回调函数
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @param[in] in_set: 属性设置消息结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note
 *    sdk 收到属性设置消息时 会调用该函数
 */
int32_t sub_device_callback_property_set(struct joylink_sdk_t *sdk, JoyProtoPropSet_t *in_set);

/**
 * @brief  属性获取 回调函数
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @param[in] in_get: 属性获取消息结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note
 *    sdk 收到属性设置消息时 会调用该函数
 */
int32_t sub_device_callback_property_get(struct joylink_sdk_t *sdk, JoyProtoPropGet_t *in_get);

/**
 * @brief  方法调用 回调函数
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @param[in] in_call: 方法调用消息结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note
 *    sdk 收到方法调用消息时 会调用该函数
 */
int32_t sub_device_callback_function_call(struct joylink_sdk_t *sdk, JoyProtoFuncCall_t *in_call);

/**
 * @brief  获取子设备信息
 * 
 * @param[in] subdeviceId: 子设备设备标识
 * @return 
 *    返回值 JoylinkRetCode_E
 */
JoyProtoSubDevice_t *sub_device_get_info(char *subdeviceId);

/**
 * @brief  子设备属性上报
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @param[in] subdeviceId: 子设备设备标识
 * @return 
 *    返回值 JoylinkRetCode_E
 */
int32_t sub_device_properties_post(struct joylink_sdk_t *sdk, char *subdeviceId);

/**
 * @brief  子设备事件上报
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @param[in] subdeviceId: 子设备设备标识
 * @return 
 *    返回值 JoylinkRetCode_E
 */
int32_t sub_device_event_post(struct joylink_sdk_t *sdk, char *subdeviceId);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _JOYLINK_SUB_DEVICE_H_ */
