/*Copyright (c) 2015-2050, JD All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License. */

/* --------------------------------------------------
 * @brief: joylink 协议 头文件，提供外部使用的相关定义和接口
 *
 * @version: 1.0
 *
 * --------------------------------------------------
 */

#ifndef _JOYLINK_PROTOCOL_H_
#define _JOYLINK_PROTOCOL_H_
#include <stdbool.h>
#include "joylink_stdint.h"
#include "joylink.h"

#ifdef __cplusplus
extern "C"{
#endif /* __cplusplus */

/**
 * @brief   宏定义 属性个数最大值，方法调用入参最大个数
 *
 */
#define JOYLINK_PROTO_PROP_MAX_NUM        (64)
#define JOYLINK_PROTO_FUNC_IN_MAX_NUM     (64)

/**
 * @brief   对象空间宏定义
 *
 */
#define JOYLINK_PROTO_OBJ_NAME_DEV      ("device")
#define JOYLINK_PROTO_OBJ_NAME_EDGE     ("edge")
#define JOYLINK_PROTO_OBJ_NAME_GROUP    ("group")

/**
 * @brief   内置服务名称宏定义
 *
 */
#define JOYLINK_PROTO_SERVICE_AGENT    ("connection_agent")
#define JOYLINK_PROTO_SERVICE_OTA      ("ota")

/**
 * @brief   connection agent 服务内置子设备操作方法
 *
 */
#define JOYLINK_PROTO_AGENT_FUNC_ADD    ("connection_agent.add-devices")
#define JOYLINK_PROTO_AGENT_FUNC_DEL    ("connection_agent.delete-devices")
#define JOYLINK_PROTO_AGENT_FUNC_UPDATE ("connection_agent.update-devices")
#define JOYLINK_PROTO_AGENT_FUNC_GET    ("connection_agent.get-devices")

/**
 * @brief   connection agent 服务内置 子设备上线下关键字
 *
 */
#define JOYLINK_PROTO_AGENT_EVENT_ONLINE    ("connection_agent.online")
#define JOYLINK_PROTO_AGENT_EVENT_OFFLINE   ("connection_agent.offline")
#define JOYLINK_PROTO_AGENT_EVENT_ADD_DEV   ("connection_agent.add-devices")
#define JOYLINK_PROTO_AGENT_EVENT_DEL_DEV   ("connection_agent.delete-devices")

/**
 * @brief   ota 服务 内置属性、方法和事件
 *
 */
#define JOYLINK_PROTO_OTA_AUTO_UPDATE       ("ota.auto-update")
#define JOYLINK_PROTO_OTA_FUNC_OTA          ("ota.ota")
#define JOYLINK_PROTO_OTA_FUNC_CANCEL_TASK  ("ota.cancel-task")
#define JOYLINK_PROTO_OTA_FUNC_QUERY_STATE  ("ota.query-ota-state")
#define JOYLINK_PROTO_OTA_FUNC_EXEC_TASK    ("ota.exec-task")
#define JOYLINK_PROTO_OTA_EVT_PROGRESS      ("ota.ota-state-changed")
#define JOYLINK_PROTO_OTA_EVT_VERSION       ("ota.ota-version-changed")
#define JOYLINK_PROTO_OTA_TASK_ID           ("task-id")
#define JOYLINK_PROTO_OTA_OBJECT_ID         ("object-id")
#define JOYLINK_PROTO_OTA_VERSION           ("version")
#define JOYLINK_PROTO_OTA_PKG_NAME          ("package-name")
#define JOYLINK_PROTO_OTA_DIFF_BASE_VER     ("diff-base-version")
#define JOYLINK_PROTO_OTA_OTA_LIST          ("ota.ota-list")

/**
 * @brief  设备ota属性实例名宏定义
 * 
 */
#define JOYLINK_PROTO_OTA_PROP_MODEL        ("ota.model")
#define JOYLINK_PROTO_OTA_PROP_AC_DIFF      ("ota.accept-diff")
#define JOYLINK_PROTO_OTA_PROP_SIG_ALG      ("ota.signature-algorithm")
#define JOYLINK_PROTO_OTA_PROP_LIST         ("ota.ota-list")

/**
 * @brief  通用消息主题种类 枚举定义
 */
typedef enum {
    JOY_PROTO_TOPIC_AUTH_POST,
    JOY_PROTO_TOPIC_AUTH_POST_RES,

    JOY_PROTO_TOPIC_THINGMODEL_POST,
    JOY_PROTO_TOPIC_THINGMODEL_POST_RES,

    JOY_PROTO_TOPIC_EVT_ONLINE,
    JOY_PROTO_TOPIC_EVT_OFFLINE,

    JOY_PROTO_TOPIC_HB_POST,
    JOY_PROTO_TOPIC_HB_POST_RES,
    JOY_PROTO_TOPIC_HB_SET,

    JOY_PROTO_TOPIC_EVT_POST,

    JOY_PROTO_TOPIC_PROP_SET,
    JOY_PROTO_TOPIC_PROP_SET_RES,
    JOY_PROTO_TOPIC_PROP_GET,
    JOY_PROTO_TOPIC_PROP_GET_RES,
    JOY_PROTO_TOPIC_PROP_POST,

    JOY_PROTO_TOPIC_FUNC_CALL,
    JOY_PROTO_TOPIC_FUNC_CALL_RES,

    JOY_PROTO_TOPIC_REG,
    JOY_PROTO_TOPIC_REG_RES,

    JOY_PROTO_TOPIC_NTP_REQ,
    JOY_PROTO_TOPIC_NTP_REQ_RES,
} JOY_PROTO_TOPIC_E;

/**
 * @brief  通用消息主题 类型定义
 */
typedef int32_t  JOY_PROTO_TOPIC_T;

/**
 * @brief  协议 键-值对 结构体定义
 */
typedef struct {
    char        *key;
    char        *value; // cJSON string
} JoyProtoKeyValue_t;

/**
 * @brief  物模型(thing-model)消息主题 物模型结构体定义
 */
typedef struct {
    char   *id;
    char   *version; // thing-model-version
} JoyProtoThingModel_t;

/**
 * @brief  物模型(thing-model)消息主题 物模型上报结构体
 */
typedef struct {
    char                    *deviceId;
    uint32_t                timestamp;
    char                    *messageId;
    JoyProtoThingModel_t    thing_model;
} JoyProtoThingModelPost_t;

/**
 * @brief  物模型(thing-model)消息主题 物模型上报响应结构体
 */
typedef struct {
    char                *deviceId;
    uint32_t            timestamp;
    char                *messageId;
    int32_t             code;
    char                *message;
} JoyProtoThingModelPostRes_t;

/**
 * @brief  属性(properties)消息主题 属性设置 消息体
 */
typedef struct {
    char                *deviceId;
    uint32_t            timestamp;
    char                *messageId;
    uint16_t            version;
    int32_t             prop_num;
    JoyProtoKeyValue_t  **properties;
} JoyProtoPropSet_t;

/**
 * @brief  属性(properties)消息主题 属性设置响应 消息体
 */
typedef struct {
    char                *deviceId;
    uint32_t            timestamp;
    char                *messageId;
    uint16_t            version;
    int32_t             code;
    char                *message;
    int32_t             prop_num;
    JoyProtoKeyValue_t  **properties;
} JoyProtoPropSetRes_t;

/**
 * @brief  属性(properties)消息主题 属性获取 消息体
 */
typedef struct {
    char                *deviceId;
    uint32_t            timestamp;
    char                *messageId;
    int32_t             prop_num;
    JoyProtoKeyValue_t  **properties;
} JoyProtoPropGet_t;

/**
 * @brief  属性(properties)消息主题 属性响应 消息体
 */
typedef JoyProtoPropSetRes_t JoyProtoPropGetRes_t;

/**
 * @brief  属性(properties)消息主题 属性上报 消息体
 */
typedef JoyProtoPropSet_t    JoyProtoPropPost_t;

/**
 * @brief   属性(properties)消息主题 释放属性设置响应结构体 成员变量的内存空间
 *
 * @param[in] in_set: 属性设置结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @see None.
 * @note 只释放其中成员变量的内存空间
 */
int32_t joylink_proto_free_prop_set_res(JoyProtoPropSetRes_t *in_res);

/**
 * @brief   属性(properties)消息主题 释放属性获取响应结构体 成员变量的内存空间
 *
 * @param[in] in_res: 属性获取响应结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E 
 * @see None.
 * @note 只释放其中成员变量的内存空间
 */
int32_t joylink_proto_free_prop_get_res(JoyProtoPropGetRes_t *in_res);

/**
 * @brief   属性(properties)消息主题 释放属性上报结构体 成员变量的内存空间
 *
 * @param[in] in_post: 属性上报结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E 
 * @see None.
 * @note 只释放其中成员变量的内存空间
 */
int32_t joylink_proto_free_prop_post(JoyProtoPropPost_t *in_post);

/**
 * @brief  方法(functions)消息主题 方法调用 消息体 functions成员定义
 */
typedef struct {
    char                *key;
    int32_t             in_num;
    JoyProtoKeyValue_t  **in;
} JoyProtoFuncCallFunc_t;

/**
 * @brief  方法(functions)消息主题 方法调用 消息体
 */
typedef struct {
    char                    *deviceId;
    uint32_t                timestamp;
    char                    *messageId;
    int32_t                 func_num;
    JoyProtoFuncCallFunc_t  **functions;
} JoyProtoFuncCall_t;

/**
 * @brief  方法(functions)消息主题 方法调用响应 消息体 functions 成员变量
 */
typedef struct {
    char                *key;
    int32_t             out_num;
    JoyProtoKeyValue_t  **out;
} JoyProtoFuncCallResFunc_t;

/**
 * @brief  方法(functions)消息主题 方法调用响应 消息体
 */
typedef struct {
    char                        *deviceId;
    uint32_t                    timestamp;
    char                        *messageId;
    int32_t                     code;
    char                        *message;
    int32_t                     func_num;
    JoyProtoFuncCallResFunc_t   **functions;
} JoyProtoFuncCallRes_t;

/**
 * @brief   方法(functions)消息主题 释放方法调用响应结构体 成员变量的内存空间
 *
 * @param[in] in_call: 方法调用结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E 
 * @see None.
 * @note 只释放其中成员变量的内存空间
 */
int32_t joylink_proto_free_func_call_res(JoyProtoFuncCallRes_t *in_res);

/**
 * @brief  事件(events)消息主题  上下线消息体
 */
typedef struct {
    char                    *deviceId;
    uint32_t                timestamp;
    char                    *messageId;
    bool                    is_online;
    JoyProtoThingModel_t    thing_model;
} JoyProtoEvtOnlineStatus_t;

/**
 * @brief  事件(events)消息主题  上报消息体 events 成员
 */
typedef struct {
    char                *key;
    int32_t             param_num;
    JoyProtoKeyValue_t  **parameters;    
}JoyProtoEvtPostEvt_t;

/**
 * @brief  事件(events)消息主题  上报消息体
 */
typedef struct {
    char                    *deviceId;
    uint32_t                timestamp;
    char                    *messageId;
    int32_t                 evt_num;
    JoyProtoEvtPostEvt_t    **events;
} JoyProtoEvtPost_t;

/**
 * @brief   事件(events)消息主题 释放事件上报结构体 成员变量的内存空间
 *
 * @param[in] in_post: 事件上报结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E 
 * @see None.
 * @note 只释放其中成员变量的内存空间
 */
int32_t joylink_proto_free_evt_post(JoyProtoEvtPost_t *in_post);

/**
 * @brief   键值对 解析布尔值
 *
 * @param[in] in_value: JoyProtoKeyValue_t 中value jSON 字符串指针
 * @param[in] out_val: 指向解析结果的指针
 * @return 
 *    返回值 JoylinkRetCode_E 
 * @see None.
 * @note None.
 */
int32_t joylink_proto_keyvalue_parse_bool(char *in_value, bool *out_val);

/**
 * @brief   键值对 打包布尔值
 *
 * @param[in] in_bool: 待打包的bool值
 * @return 
 *    返回 打包后 JoyProtoKeyValue_t 中value jSON 字符串指针 
 * @see None.
 * @note None.
 */
char *joylink_proto_keyvalue_pack_bool(bool in_bool);

/**
 * @brief   键值对 解析 int32_t
 *
 * @param[in] in_value: JoyProtoKeyValue_t 中value jSON 字符串指针
 * @param[in] out_val: 指向解析结果的指针
 * @return 
 *    返回值 JoylinkRetCode_E 
 * @see None.
 * @note None.
 */
int32_t joylink_proto_keyvalue_parse_int32(char *in_value, int32_t *out_val);

/**
 * @brief   键值对 打包 int32_t
 *
 * @param[in] in_int32: 待打包的int32值
 * @return 
 *    返回 打包后 JoyProtoKeyValue_t 中value jSON 字符串指针 
 * @see None.
 * @note None.
 */
char *joylink_proto_keyvalue_pack_int32(int32_t in_int32);

/**
 * @brief   键值对 解析 double
 *
 * @param[in] in_value: JoyProtoKeyValue_t 中value jSON 字符串指针
 * @param[in] out_val: 指向解析结果的指针
 * @return 
 *    返回值 JoylinkRetCode_E 
 * @see None.
 * @note None.
 */
int32_t joylink_proto_keyvalue_parse_double(char *in_value, double *out_val);

/**
 * @brief   键值对 打包 double
 *
 * @param[in] in_double: 待打包的double值
 * @return 
 *    返回 打包后 JoyProtoKeyValue_t 中value jSON 字符串指针 
 * @see None.
 * @note None.
 */
char *joylink_proto_keyvalue_pack_double(double in_double);

/**
 * @brief   键值对 解析 string
 *
 * @param[in] in_value: JoyProtoKeyValue_t 中value jSON 字符串指针
 * @param[in] out_val: 指向解析结果的指针
 * @return 
 *    返回 解析后的值 
 * @see None.
 * @note None.
 */
char *joylink_proto_keyvalue_parse_string(char *in_value);

/**
 * @brief   键值对 打包 string
 *
 * @param[in] in_string: 待打包的string
 * @return 
 *    返回 打包后 JoyProtoKeyValue_t 中value jSON 字符串指针 
 * @see None.
 * @note None.
 */
char *joylink_proto_keyvalue_pack_string(char *in_string);

/**
 * @brief  自动注册(register) 请求结构体 devcies成员 扩展项定义
 */
typedef struct {
    char        *key;
    char        *value;
    char        *desc;
} JoyProtoRegReqDevExt_t;

/**
 * @brief  自动注册(register) 请求结构体 devcies 成员结构体定义
 */
typedef struct {
    char                    *hardwareId;
    char                    *vendor;
    char                    *protocol;
    char                    *product;
    char                    *model;
    int32_t                 ext_num;
    JoyProtoRegReqDevExt_t  **extensions;
} JoyProtoRegReqDev_t;

/**
 * @brief  自动注册(register) 请求结构体
 */
typedef struct {
    char                    *deviceId;
    uint32_t                timestamp;
    char                    *messageId;
    int32_t                 dev_num;
    JoyProtoRegReqDev_t     **devices;
    char                    cerType[8];
} JoyProtoRegReq_t;

/**
 * @brief  自动注册(register) 请求响应结构体 devcies成员 配置项定义
 */
typedef struct {
    char        *name;
    char        *type;
    char        *content;
} JoyProtoRegReqResDevCfg_t;

/**
 * @brief  自动注册(register) 请求响应结构体 devcies成员结构体定义
 */
typedef struct {
    char                        *deviceId;
    char                        *hardwareId;
    uint32_t                    cfg_num;
    JoyProtoRegReqResDevCfg_t   **config;
} JoyProtoRegReqResDev_t;

/**
 * @brief  自动注册(register) 请求响应结构体
 */
typedef struct {
    char                        *deviceId;
    uint32_t                    timestamp;
    char                        *messageId;
    int32_t                     code;
    char                        *message;
    uint32_t                    dev_num;
    JoyProtoRegReqResDev_t      **devices;
} JoyProtoRegReqRes_t;

/**
 * @brief   connection agent 子设备定义
 */
typedef struct {
    char    *name;
    char    *deviceId;
    char    *description;
    char    *modelId;
    char    *serviceName;
    char    *hardwareId;
    char    *protocolName;
    char    *protocolSpec;
}JoyProtoSubDevice_t;

/**
 * @brief   内置服务(connection agent) 释放 子设备结构体成员变量
 *
 * @param[in] in_dev: 子设备信息结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E 
 * @see None.
 * @note 只释放其中成员变量的内存空间
 */
int32_t joylink_proto_free_sub_dev(JoyProtoSubDevice_t *in_dev);

/**
 * @brief   内置服务(connection agent) 拷贝 子设备结构体成员变量
 *
 * @param[in] dest_dev: 目的子设备信息结构体指针
 * @param[in] src_dev: 源子设备信息结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E 
 * @see None.
 * @note 只释放其中成员变量的内存空间
 */
int32_t joylink_proto_copy_sub_dev(JoyProtoSubDevice_t *dest_dev, JoyProtoSubDevice_t *src_dev);

/**
 * @brief   connection agent 子设备信息
 */
typedef struct {
    int32_t             dev_num;
    JoyProtoSubDevice_t **subDev;
}JoyProtoSubDeviceInfo_t;

/**
 * @brief   内置服务(connection agent) 释放 子设备信息结构体成员变量
 *
 * @param[in] in_info: 子设备信息结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E 
 * @see None.
 * @note 只释放其中成员变量的内存空间
 */
int32_t joylink_proto_free_sub_dev_info(JoyProtoSubDeviceInfo_t *in_info);

/**
 * @brief   内置服务(connection agent) 键值对 解析 connection agent 子设备信息
 *
 * @param[in] in_value: JoyProtoKeyValue_t 中value jSON 字符串指针
 * @param[in] out_info: 指向解析结果的指针
 * @return 
 *    返回值 JoylinkRetCode_E 
 * @see None.
 * @note None.
 */
int32_t joylink_proto_keyvalue_parse_subdevinfo(char *in_value, JoyProtoSubDeviceInfo_t *out_info);

/**
 * @brief   内置服务(connection agent) 键值对 打包 connection agent 子设备信息
 *
 * @param[in] in_info: connection agent 子设备信息
 * @return 
 *    返回 打包后 JoyProtoKeyValue_t 中value jSON 字符串指针 
 * @see None.
 * @note None.
 */
char *joylink_proto_keyvalue_pack_subdevinfo(JoyProtoSubDeviceInfo_t *in_info);

/**
 * @brief   内置服务(connection agent) 创建操作子设备方法响应
 *
 * @param[in] func_key: 操作方法的名字
 * @param[in] dev: 子设备信息指针，除get-devices外其余填NULL
 * @param[in] code: 内置方法 返回值
 * @param[in] message: 内置方法 返回信息字符串指针
 * @return 
 *    返回 函数调用响应functions成员结构体指针 
 * @see None.
 * @note None.
 */
JoyProtoFuncCallResFunc_t *joylink_proto_agent_create_opsub_func_Res(
                            char *func_key, 
                            JoyProtoSubDeviceInfo_t *dev,
                            int32_t code, 
                            char *message);

/**
 * @brief   内置服务(connection agent) 创建子设备在线状态事件
 *
 * @param[in] evt_key: 事件的名字
 * @param[in] dev: 子设备信息指针
 * @return 
 *    返回 事件上报events成员结构体指针 
 * @see None.
 * @note None.
 */
JoyProtoEvtPostEvt_t *joylink_proto_agent_create_online_event(
                            char *evt_key, 
                            JoyProtoSubDeviceInfo_t *dev);

/**
 * @brief   ota 创建ota方法调用响应
 *
 * @param[in] func_key: 方法调用的实例名
 * @return 
 *    返回 函数调用响应functions成员结构体指针 
 * @see None.
 * @note None.
 */
JoyProtoFuncCallResFunc_t *joylink_proto_ota_create_func_res(char *func_key);

/**
 * @brief  ota 当前状态枚举
 */
typedef enum {
    JOY_OTA_ST_PREPARE,           // 正在准备
    JOY_OTA_ST_CF_DOWNLOAD,       // 等待用户确认下载
    JOY_OTA_ST_DOWNLOADIND,       // 正在下载
    JOY_OTA_ST_DOWNLOAD_DONE,     // 下载完成
    JOY_OTA_ST_CF_INSTALL,        // 等待用户确认安装
    JOY_OTA_ST_INSTALLING,        // 正在安装
    JOY_OTA_ST_INSTALL_DONE,      // 完成
    JOY_OTA_ST_FAILURE,           // 失败
} JOY_PROTO_OTA_STATE_E;
typedef int32_t  JOY_PROTO_OTA_STATE_T;

/**
 * @brief  ota 状态错误码
 */
typedef enum {
    JOY_OTA_ERROR_NONE,             // 无异常
    JOY_OTA_ERROR_TASK_CL,          // OTA任务已取消
    JOY_OTA_ERROR_DOWNLOAD_CL,      // 用户取消下载
    JOY_OTA_ERROR_INSTALL_CL,       // 用户取消安装
    JOY_OTA_ERROR_DPG_VER_FAIL,     // 差分包基包验证失败
    JOY_OTA_ERROR_DOWNLOAD_FAIL,    // 下载失败
    JOY_OTA_ERROR_SIG_VER_FAIL,     // 签名验证失败
    JOY_OTA_ERROR_DPG_RVT_FAIL,     // 差分包还原失败
    JOY_OTA_ERROR_INSTALL_FAIL,     // 安装失败
} JOY_PROTO_OTA_ERROR_E;
typedef int32_t  JOY_PROTO_OTA_ERROR_T;

/**
 * @brief   ota 创建ota方法调用 状态查询响应
 *
 * @param[in] func_key: 方法调用的实例名
 * @param[in] task_id: 任务标识
 * @param[in] state: 当前状态
 * @param[in] e_code: 错误码
 * @param[in] progress: 当前进度百分比
 * @return 
 *    返回 函数调用响应functions成员结构体指针 
 * @see None.
 * @note None.
 */
JoyProtoFuncCallResFunc_t *joylink_proto_ota_create_func_query_res(char *func_key,
                                                             char *task_id,
                                                             int32_t state,
                                                             JOY_PROTO_OTA_ERROR_T e_code,
                                                             int8_t progress);

/**
 * @brief   ota ota-list 属性 数组成员定义
 */
typedef struct {
    char        *package_name;
    int32_t     version;
    int32_t     diff_base_version;
} JoyProtoOtaListMember_t;

/**
 * @brief   ota ota-list 属性 结构体定义
 */
typedef struct {
    int32_t                 list_len;
    JoyProtoOtaListMember_t **members;
} JoyProtoOtaList_t;

/**
 * @brief   ota 键值对 打包 ota-list 信息
 *
 * @param[in] in_list: ota-list 数组
 * @return 
 *    返回 打包后 JoyProtoKeyValue_t 中value jSON 字符串指针 
 * @see None.
 * @note None.
 */
char *joylink_proto_keyvalue_pack_ota_list(JoyProtoOtaList_t *in_list);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _JOYLINK_PROTOCOL_H_  */
